Package: DiceDesign
Type: Package
Title: Designs of Computer Experiments
Version: 1.10
Date: 2023-11-30
Author: Jessica Franco, Delphine Dupuy, Olivier Roustant, 
        Patrice Kiener, Guillaume Damblin and Bertrand Iooss.
Maintainer: Celine Helbert <Celine.Helbert@ec-lyon.fr>
Description: Space-Filling Designs and space-filling criteria (distance-based and uniformity-based), with emphasis to computer experiments; <doi:10.18637/jss.v065.i11>.
License: GPL-3
Depends: R (>= 2.10)
Suggests: rgl, randtoolbox, lattice
Encoding: UTF-8
NeedsCompilation: yes
LazyData: true
Packaged: 2023-12-07 08:35:30 UTC; chelbert
Repository: CRAN
Date/Publication: 2023-12-07 12:00:10 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:13:50 UTC; unix
Archs: DiceDesign.so.dSYM

Package: DiceDesign
Type: Package
Title: Designs of Computer Experiments
Version: 1.10
Date: 2023-11-30
Author: Jessica Franco, Delphine Dupuy, Olivier Roustant, 
        Patrice Kiener, Guillaume Damblin and Bertrand Iooss.
Maintainer: Celine Helbert <Celine.Helbert@ec-lyon.fr>
Description: Space-Filling Designs and space-filling criteria (distance-based and uniformity-based), with emphasis to computer experiments; <doi:10.18637/jss.v065.i11>.
License: GPL-3
Depends: R (>= 2.10)
Suggests: rgl, randtoolbox, lattice
Encoding: UTF-8
NeedsCompilation: yes
LazyData: true
Packaged: 2023-12-07 08:35:30 UTC; chelbert
Repository: CRAN
Date/Publication: 2023-12-07 12:00:10 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 10:17:34 UTC; unix
Archs: DiceDesign.so.dSYM

useDynLib(DiceDesign, .registration = TRUE, .fixes = "C_") 

export(
 coverage,
 discrepancyCriteria,
 discrepESE_LHS,
 discrepSA_LHS,
 dmaxDesign, 
 factDesign,
 faureprimeDesign,
 lhsDesign,
 maximinESE_LHS,
 maximinSA_LHS,
 meshRatio,
 mindist,
 mstCriteria,
 nolhDesign,
 nolhdrDesign,
 olhDesign,
 phiP,
 rss2d,
 rss3d,
 runif.faure,
 scaleDesign,
 straussDesign,
 unif.test.quantile,
 unif.test.statistic,
 unscaleDesign,
 wspDesign,
 xDRDN
)

import(graphics, grDevices, stats)

unif.test.quantile <- function(type, n, alpha=0.05) {

	# statistic upper tail percentage points
	# see D'Agostino and Stephens "Goodness-of-fit techniques", 1986

if (!is.element(type, c("greenwood", "qm", "ks", "V", "cvm"))) stop("The goodness-of-fit test must be one of: Greenwood, Quesenberry-Miller, Kolmogorov-Smirnov, V = (D+) +  (D-), or Cramer-Von Mises")

if (length(alpha)!=1) {
	stop("At this stage, alpha must be a single number")
} else if (!is.element(alpha, c(0.1, 0.05, 0.025, 0.01))) {
	stop("Significance level must be a single number among 0.1, 0.05, 0.025 or 0.01")
}

index <- is.element(c(0.1, 0.05, 0.025, 0.01), alpha)

if (type=="greenwood") {
	# quantile of n*G(n)
	if (n>500) {
		q <- qnorm(1-alpha/2, mean=2*n/(n+2), sd=sqrt(4/n))
	} else {
                # greenwood.table is loaded via sysdat.rda
		f <- approxfun(greenwood.table[,1], greenwood.table[,c(2,3,4,5)[index]])
		q <- f(n)
	}
	q <- q/n		# quantile of G(n)
} else if (type=="QM") {
	stop("no default value for Quesenberry and Miller statistic")
} else if (type=="ks") {
	q <- c(1.224, 1.358, 1.480, 1.628)[index]/(sqrt(n) + 0.12 + 0.11/sqrt(n))
} else if (type=="V") {
	q <- c(1.620, 1.747, 1.862, 2.001)[index]/(sqrt(n) + 0.155 + 0.24/sqrt(n))
} else if (type=="cvm") {
	q <- c(0.347, 0.461, 0.581, 0.743)[index]/(1+1/n) + 0.4/n - 0.6/n^2
}
	
return(q)
}

rss2d <- function(design, lower, upper, gof.test.type="greenwood", gof.test.stat=NULL, transform=NULL, n.angle=360, graphics=1, trace=TRUE, lines.lwd=1, lines.lty="dotted", ...)  {

design <- as.matrix(design)
n <- dim(design)[1]
d <- dim(design)[2]


	# some arguments checks
	
if (!is.element(gof.test.type, c("greenwood", "qm", "ks", "V", "cvm"))) stop("The goodness-of-fit test must be one of: Greenwood, Quesenberry-Miller, Kolmogorov-Smirnov, V = (D+) +  (D-), or Cramer-Von Mises")

if ((length(lower)!=d) & (length(upper)!=d)) stop("lower and upper must be d-dimensional vectors")


	# domain transformation to [-1,1]^d 
	
for (j in 1:d) {
	design.col <- design[,j]
	design.col.min <- min(design.col)
	if (design.col.min < lower[j]) stop('the minimum of design values is not compatible with lower')
	design.col.max <- max(design.col)
	if (design.col.max > upper[j]) stop('the maximum of design values is not compatible with upper')
	design.col <- 2*((design.col - lower[j])/(upper[j] - lower[j]) - 0.5)
	design[,j] <- design.col
}


	# compute the subdivision of the (half) circle in cartesian coordinates

theta.degree <- seq(from=0, to=180, length=n.angle+1)
theta.degree <- theta.degree[1:n.angle]
theta.degree <- as.matrix(theta.degree)
theta <- theta.degree*2*pi/360
cos.theta <- cos(theta)
sin.theta <- sin(theta)
n.theta <- length(theta)
subdiv.halfcircle <- cbind(cos.theta, sin.theta)


	# loop over all pairs of dimensions

global.stat <- matrix(NA, d, d)
global.stat.max <- 0

if (trace) {
	cat("\n2D Radial Scanning Statistic (RSS) with ", toupper(gof.test.type), " statistic\n", sep="")
	cat("Discretization step (in degree) : ", 180/n.angle, sep="")
	cat("\n\nMaximum of RS statistic values (global statistic) per pair of dimensions")
}


print.out <- data.frame(global.stat = rep(NA, (d*(d-1))%/%2))
meter <- 0

for (i in 1:(d-1)) {
	x <- design[,i]

	for (j in ((i+1):d)) {
		y <- design[,j]
		
			# compute anglewise statistic
		
				
				# 1st step : compute the matrix of the F(projected points onto Vect(cos.theta, sin.theta))
		out <- .C("C_rss2Dloop", as.double(x), as.double(y), as.double(cos.theta), as.double(sin.theta), as.integer(n), as.integer(n.theta), ans=double(n * n.theta), PACKAGE="DiceDesign") 
		F.projections <- matrix(out$ans, n, n.theta)

				# 2nd step : for each angle, compute the statistic 
				# In future version, should be computed inside the C loop
		anglewise.stat.ij <- matrix(NA, n.theta, 1)
		for (angle in 1:n.theta) {
			anglewise.stat.ij[angle] <- unif.test.statistic(x=F.projections[,angle], type=gof.test.type, transform=transform)
		}
			

			# compute the worst value over all angles and store it
		global.stat.ij <- max(anglewise.stat.ij)      
		global.stat[i,j] <- global.stat[j,i] <- global.stat.ij
		
		if (global.stat.ij > global.stat.max) {
			global.stat.max <- global.stat.ij
			pair.worst <- c(i,j)
			anglewise.stat <- anglewise.stat.ij
		} 
		
		meter <- meter + 1
		print.out[meter, 1] <- global.stat.ij
		name.current <- paste("(", i, ",", j, ")", sep="")
		row.names(print.out)[meter] <- name.current
		if (trace) cat("\n", name.current, " ", global.stat.ij, sep="")
		
	} # end loop j
} # end loop i
	
if (trace) cat("\n\n")

	
	# rss curve
rss.curve.x <- anglewise.stat*subdiv.halfcircle[,1]
rss.curve.y <- anglewise.stat*subdiv.halfcircle[,2]


	# statistic upper tail percentage points
	# see D'Agostino and Stephens "Goodness-of-fit techniques", 1986

if (is.null(gof.test.stat)) {
	gof.test.stat <- unif.test.quantile(type=gof.test.type, n=n, alpha=0.05)
}

# compute worst direction for the worst case
design.names <- names(as.data.frame(design))

design <- design[ , pair.worst] 
design.names <- design.names[pair.worst]

anglewise.stat.max <- max(anglewise.stat)
index.max <- which.max(anglewise.stat)
cos.theta.max <- subdiv.halfcircle[index.max, 1]
sin.theta.max <- subdiv.halfcircle[index.max, 2]
dir.max <- c(cos.theta.max, sin.theta.max)


	# --------------------
	# graphics if required
	# --------------------



if (graphics>=0) {

	if (is.element(graphics, c(0,1))) {
		par(mfrow=c(1,2+graphics))
		plot(design, xlim=c(-1,1), ylim=c(-1,1), 
		             xlab=design.names[1], ylab=design.names[2])
	}

	
		# draw the rss curve
	if (graphics>0) {
		rx <- c(rss.curve.x, -rss.curve.x, rss.curve.x[1])
		ry <- c(rss.curve.y, -rss.curve.y, rss.curve.y[1])
		graph.size <- max(abs((anglewise.stat.max)*dir.max), gof.test.stat)*1.05
		plot(rx, ry, xlim=c(-graph.size,graph.size), ylim=c(-graph.size, graph.size), 
									 xlab="", ylab="", ...)

			# draw the circle with radius equal to the threshold at significance level 5%
		theta_aux <- seq(from=0, to=2*pi+0.1,by=0.1)
		lines(gof.test.stat*cos(theta_aux), gof.test.stat*sin(theta_aux))

			# draw the coordinate axis in dotted lines
		abline(h=0, v=0, lty=lines.lty, col="black", lwd=lines.lwd)
	}
 	
 	if (is.element(graphics, c(0,1))) {
 		plot(design, xlim=c(-1,1), ylim=c(-1,1), 
		             xlab=design.names[1], ylab=design.names[2])
 		projections <- design %*% dir.max
		points(projections*dir.max[1], projections*dir.max[2], pch=20, col="red")
		if (cos.theta.max==0) {
			lines(c(0,0), c(-1,1), col="red")
		} else lines(c(-1,1), c(-1,1)*sin.theta.max/cos.theta.max, col="red")
		for (i in 1:n) lines(c(design[i,1], projections[i]*cos.theta.max), c(design[i,2], projections[i]*sin.theta.max), lty=lines.lty, lwd=lines.lwd)
 	}
	
	par(mfrow=c(1,1))
}

return(list(global.stat=global.stat, worst.case=pair.worst, worst.dir=dir.max, stat=as.numeric(anglewise.stat), angle=as.numeric(theta), curve=cbind(c(rss.curve.x,-rss.curve.x), c(rss.curve.y,-rss.curve.y)),  gof.test.stat=gof.test.stat))
}

unif.test.statistic <- function(x, type, transform=NULL) {


if (!is.null(transform)) {
	if (transform=="spacings") {
		ordered.sample <- sort(x)
		ordered.sample <- c(0, ordered.sample, 1)
		spacings <- diff(ordered.sample)
		ordered.spacings <- sort(spacings)
		n <- length(x)
		S0 <- (n+1)*ordered.spacings[1]
		S <- (n+1-(1:n))*diff(ordered.spacings)
		V <- cumsum(c(S0, S))
		x <- V
	}
}

if (type=="greenwood") {
	
	ordered.sample <- sort(x)
	ordered.sample <- c(0, ordered.sample, 1)
	spacings <- diff(ordered.sample)
	stat <- sum(spacings^2)
	
} else if (type=="spacings.max") {
	
	ordered.sample <- sort(x)
	ordered.sample <- c(0, ordered.sample, 1)
	spacings <- diff(ordered.sample)
	n <- length(x)
	stat <- max(abs(spacings-1/n))
}

else if (type=="qm") {    # Quesenberry-Miller

	ordered.sample <- sort(x)
	n.sample <- length(x)
	ordered.sample <- c(0, ordered.sample, 1)
	spacings <- diff(ordered.sample)
	stat <- sum(spacings^2)+sum(spacings[1:n.sample]*spacings[2:(n.sample+1)])

} else if (type=="ks") {

	ordered.sample <- sort(x)
	n.sample <- length(x)
	Dplus <- max((1:n.sample)/n.sample - ordered.sample)
	Dminus <- max(ordered.sample - (0:(n.sample-1))/n.sample)
	stat <- max(Dplus, Dminus)

} else if (type=="V") {

	ordered.sample <- sort(x)
	n.sample <- length(x)
	Dplus <- max((1:n.sample)/n.sample - ordered.sample)
	Dminus <- max(ordered.sample - (0:(n.sample-1))/n.sample)
	stat <- Dplus + Dminus

} else if (type=="cvm") {

	ordered.sample <- sort(x)
	n.sample <- length(x)
	stat <- sum((ordered.sample - (seq(from = 1, to = (2 * n.sample - 1), by = 2)/(2 * n.sample)))^2) + 1/(12 * n.sample)

} else stop("The goodness-of-fit test must be one of: Greenwood, Quesenberry-Miller, Kolmogorov-Smirnov, V=D+ +  D-, or Cramer-Von Mises")


return(stat)
}

rss3d<-function(design, lower, upper, gof.test.type="greenwood", gof.test.stat=NULL, transform=NULL, n.angle=60, graphics=1, trace=TRUE){


design <- as.matrix(design)
n <- dim(design)[1]
d <- dim(design)[2]

#if (ncol(design)!=3) stop("'design' must have 3 columns")
#if ((length(lower)!=3) & (length(upper)!=3)) stop("lower and upper must be 3-dimensional vectors")
#dimension <- 3

	# some arguments checks

if (d < 3) stop("rss3d is for d-dimensional designs with d >= 3")	
if (!is.element(gof.test.type, c("greenwood", "qm", "ks", "V", "cvm"))) stop("The goodness-of-fit test must be one of: Greenwood, Quesenberry-Miller, Kolmogorov-Smirnov, V = (D+) +  (D-), or Cramer-Von Mises")

if ((length(lower)!=d) & (length(upper)!=d)) stop("lower and upper must be d-dimensional vectors")


	# domain transformation to [-1,1]^d 

for (j in 1:d) {
	design.col <- design[,j]
	design.col.min <- min(design.col)
	if (design.col.min < lower[j]) stop('the minimum of design values is not compatible with lower')
	design.col.max <- max(design.col)
	if (design.col.max > upper[j]) stop('the maximum of design values is not compatible with upper')
	design.col <- 2*((design.col - lower[j])/(upper[j] - lower[j]) - 0.5)
	design[,j] <- design.col
}


	# angles definition

theta.degree <- seq(from=0, to=180, length=n.angle+1)
theta.degree <- theta.degree[1:n.angle]
theta.degree <- as.matrix(theta.degree)
theta <- theta.degree*2*pi/360
n.theta <- length(theta)        
cos.theta <- cos(theta); sin.theta <- sin(theta)

phi.degree <- seq(from=-90, to=90, length=n.angle+10)
phi.degree <- phi.degree[1:n.angle]
phi.degree <- as.matrix(phi.degree)
phi <- phi.degree*2*pi/360
n.phi <- length(phi)
cos.phi <- cos(phi); sin.phi <- sin(phi)


	# Loops on theta and phi to build the matrix of statistic values, one of which is implemented in C

ax <- cos.theta%*%t(cos.phi)
ay <- sin.theta%*%t(cos.phi)
az <- rep(1, n.theta)%*%t(sin.phi)

print.out <- data.frame(global.stat = rep(NA, (d*(d-1)*(d-2))%/%6))

	# loop over all triplets of dimensions

anglewise.stat.max <- anglewise.stat <- matrix(0, n.theta, n.phi)
global.stat.array <- array(NA, c(d, d, d))
global.stat.max <- 0
meter <- 0

if (trace) {
	cat("\n3D Radial Scanning Statistic (RSS) with ", toupper(gof.test.type), " statistic\n", sep="")
	cat("Discretization step (in degree) : ", 180/n.angle, sep="")
	cat("\n\nMaximum of RS statistic values (global statistic) per triplet of dimensions")
}

for (i1 in 1:(d-2)) {
	x <- design[, i1]

	for (i2 in ((i1+1):(d-1))) {
		y <- design[, i2]

		for (i3 in ((i2+1):d)) {
			z <- design[, i3]


			for (j in 1:n.phi){                  

					# 1st step : compute the matrix of the F(projected points onto Vect(cos.theta, sin.theta))
							
				out <- .C("C_rss3Dloop", as.double(x), as.double(y), as.double(z), as.double(ax), as.double(ay), as.double(az), as.integer(n), as.integer(n.theta), as.integer(n.phi), as.integer(j-1), ans=double(n * n.theta), PACKAGE="DiceDesign") 

				F.projections <- matrix(out$ans, n, n.theta)

	
					# 2nd step: test statistic computations

				for (k in 1:n.theta) {
					anglewise.stat[k,j] <- unif.test.statistic(x=F.projections[,k], type=gof.test.type, transform=transform)
				}	# end loop over theta (k)

			} # end loop over phi (j)
	
				# compute the worst value over all angles and store it
			global.stat <- max(anglewise.stat)      
			global.stat.array[i1,i2,i3] <- global.stat
		
			if (global.stat > global.stat.max) {
				global.stat.max <- global.stat
				triplet.worst <- c(i1,i2,i3)
				anglewise.stat.max <- anglewise.stat
			} 
			
			meter <- meter + 1
			print.out[meter, 1] <- global.stat
			name.current <- paste("(", i1, ",", i2, ",", i3, ")", sep="")
			row.names(print.out)[meter] <- name.current
			if (trace) cat("\n", name.current, " ", global.stat, sep="")
	
		} # end loop i3
	} # end loop i2
} # end loop i1

if (trace) cat("\n\n")
	
	# threshold at significance level 5%
	
if (is.null(gof.test.stat)) {
	gof.test.stat <- unif.test.quantile(type=gof.test.type, n=n, alpha=0.05)
}


  # Compute the worst direction for the worst case

design <- design[, triplet.worst]
x <- design[, 1]
y <- design[, 2]
z <- design[, 3]

index.max <- which.max(anglewise.stat.max)
ax.max <- ax[index.max]
ay.max <- ay[index.max]
az.max <- az[index.max] 

phi.max <- theta.max <- NA
for (j in 1:n.phi) {
  for (k in 1:n.theta) {
    if (abs(anglewise.stat.max[k,j]-global.stat.max)<1e-10) {
      phi.max <- phi[j]
      theta.max <- theta[k]
    } 			
  }
}

dir.max <- c(ax.max, ay.max, az.max)

	
	# 3D graph with package rgl ##

if (graphics > 0) {
	
  if (requireNamespace("rgl", quietly = TRUE)) {
    rgl::open3d()
    projections <- as.matrix(design) %*% dir.max
    
    rgl::plot3d(x, y, z, size=5, col="blue", xlim=c(-1,1), ylim=c(-1,1), zlim=c(-1,1), xlab="", ylab="", zlab="")
    
    a.max <- max(abs(dir.max))
    
    rgl::plot3d(c(-1,1)*ax.max/a.max, c(-1,1)*ay.max/a.max, c(-1,1)*az.max/a.max, col="red", type="l", size=2, add=TRUE)
    
    
    for (i in 1:n) {
      h <- projections[i]*dir.max - design[i,]
      if (max(abs(h)) > 1e-8) {
        lambda <- min(min((sign(h) - design[i,])/h), 1)
        rgl::plot3d(c(design[i,1], design[i,1] + lambda*h[1]), c(design[i,2], design[i,2] + lambda*h[2]), c(design[i,3], design[i,3] + lambda*h[3]), type="l", col="red", add=TRUE)
      }
      if (max(abs(projections[i]*dir.max))<=1) rgl::plot3d(projections[i]*dir.max[1], projections[i]*dir.max[2], projections[i]*dir.max[3], pch=20, col="red", size=5, add=TRUE)
    }
    par(mfrow=c(1,1))
    
  } else {
   print("Error : the package rgl is not installed")
  }
	
	
} # end of conditional block: "if graphics > 0"

return(list(stat=anglewise.stat.max, angle=data.frame(theta=theta, phi=phi), global.stat=global.stat.array, print.out=print.out, gof.test.stat=gof.test.stat, worst.case=triplet.worst, worst.dir=dir.max))

} 

############################################################################
# Elementary functions for:
# discrepSA_LHS,discrepESE_LHS, maximinSA_LHS and maximinESE_LHS
############################################################################

# EP  = Elementary Permutation

#####discrepancyTERM_L2_STAR#####
#---------------------------------------------------------------------------|
#args :  i,j : two points x_i and x_j from the design m                     |
#        m   : the design                                                   |
#depends :  both "term_oneDD" and "term_twoDD"                              |
#---------------------------------------------------------------------------|

discrepancyTERM_L2_STAR <- function(i,j,m)
{
  n<-nrow(m)
  d<-ncol(m)
  if(i!=j)
  { t<-c()
    for (l in 1:d)
    {t<-c(t,1-max(m[i,l],m[j,l]))}
    t<-(prod(t))/(n^2)  
  }
  else 
  {t<-term_oneDD(i,m)/(n^2)-((2^(1-d))/n)*term_twoDD(i,m)}
  return(t)
}

#####term_twoDD#####

term_twoDD=function(i,m)
  
{
  t2<-1-m[i,]^2
  t2<-prod(t2)
  return(t2)
}

#####term_oneDD#####

term_oneDD=function(i,m)
  
{
  t1<-1-m[i,]
  t1<-prod(t1)
  return(t1)
}


#####FUNCTION PERFORMING ELEMENTARY PERMUTATION (EP) IN LHD##
#####USED IN SA ALGORITHMS#####

#####lhs_EP#####
#---------------------------------------------------------------------------|
#args :  m = the design                                                     |
#out  :  l = list including design after EP, ligns and columns defining EP  |
#---------------------------------------------------------------------------|

lhs_EP<-function(m) 
{  
  G<-m
  if (!is.matrix(G)) G<-m[[1]]
  d<-ncol(G)
  n<-nrow(G)
  ligns<-trunc(runif(2,0,n))+1      
  column<-trunc(runif(1,0,d))+1
  x<-G[ligns[1],column]
  G[ligns[1],column]<-G[ligns[2],column]
  G[ligns[2],column]<-x 
  l<-list(G,ligns[1],ligns[2],column)   
  return(l)
} 

#####phiP_EP_ESE#####
#---------------------------------------------------------------------------|
#args :  m     : the design before the EP                                   |
#        k     : the column of lhs elementary permutation                   |
#        p     : the power in phiP criterion                                |
#out     l     : a list with new design and its mindist value               |
#---------------------------------------------------------------------------|

phiP_EP_ESE<-function(m,k,p) 
{  
  d<-ncol(m)
  n<-nrow(m)
  G<-m
  ligns<-trunc(runif(2,0,n))+1      
  x<-G[ligns[1],k]
  G[ligns[1],k]<-G[ligns[2],k]
  G[ligns[2],k]<-x 
  l<-list(phiP(G,p),G)   
  return(l)
} 


#####INTERMEDIATE FUNCTIONS#####

alpha_oneDD<-function(i1,i2,k,m)
{
  
  alpha<-(1-m[i2,k])/(1-m[i1,k])
  alpha
}

beta_oneDD<-function(i1,i2,k,m)
{
  
  beta<-(1-m[i2,k]^2)/(1-m[i1,k]^2)
  beta
}


gamma_oneDD<-function(i1,i2,k,j,m)
{
  
  gamma<-(1-max(m[i2,k],m[j,k]))/(1-max(m[i1,k],m[j,k]))
  gamma
}

#####DESIGN'S L2 STAR DISCREPANCY VALUE AFTER AN EP#####

#####discrepancyL2_EP_ESE#####
#---------------------------------------------------------------------------|
#args :  m     : the design before the EP                                   |
#        k     : the column of lhs elementary permutation                   |
#        p     : the square of the L2_star_discrepancy of m                 |
#out     l : list with dL2_2 (the square of the L2_star_discrepancy         |
#             of the EP design and the matrix corresponding to the EP design|
#depends :  term_oneDD, term_twoDD, alpha_oneDD, beta_oneDD,                |
#           gamma_oneDD, discrepancyTERM_L2_STAR                            |
#---------------------------------------------------------------------------|

discrepancyL2_EP_ESE<-function(m,k,p)
{
  G<-m
  # On genere un autre LHS a partir de M, en le perturbant de facon minimale
  i<-trunc(runif(2,0,nrow(m)))+1  
  x<-G[i[1],k]
  G[i[1],k]<-G[i[2],k]
  G[i[2],k]<-x 
  i1<-i[1]
  i2<-i[2]
  
  
  n<-nrow(m)
  d<-ncol(m)
  dL2_2<-p+(term_oneDD(i1,m)*alpha_oneDD(i1,i2,k,m))/(n^2)-beta_oneDD(i1,i2,k,m)*term_twoDD(i1,m)*((2^(1-d))/n)-discrepancyTERM_L2_STAR(i1,i1,m)
  dL2_2<-dL2_2+(term_oneDD(i2,m)/(alpha_oneDD(i1,i2,k,m)*(n^2)))-((term_twoDD(i2,m)*((2^(1-d))/n))/(beta_oneDD(i1,i2,k,m)))-discrepancyTERM_L2_STAR(i2,i2,m)
  y<-rep(0,n-2)
  w<-0
  for (j in 1:n)
  {if (j!=i1 & j!=i2)
  {
    w<-w+1
    y[w]<-gamma_oneDD(i1,i2,k,j,m)*discrepancyTERM_L2_STAR(i1,j,m)-discrepancyTERM_L2_STAR(i1,j,m)+(discrepancyTERM_L2_STAR(i2,j,m)/(gamma_oneDD(i1,i2,k,j,m)))-discrepancyTERM_L2_STAR(i2,j,m)
  }
   
  }
  
  y<-sum(y)
  dL2_2<-dL2_2+2*y
  l<-list(dL2_2,G)
  return(l)
}

#####DESIGN'S L2 DISCREPANCY VALUE AFTER AN EP#####


#####discrepancyL2_EP#####
#---------------------------------------------------------------------------|
#args :  m     : the design before the EP                                   |
#        i1,i2 : the two ligns of the lhs elementary permutation            |
#        k     : the column of lhs elementary permutation                   |
#        p     : the square of the L2_star_discrepancy of m                 |
#out     dL2_2 : the square of the L2_star_discrepancy of the EP design     |
#depends :  term_oneDD, term_twoDD, alpha_oneDD, beta_oneDD,                |
#           gamma_oneDD, discrepancyTERM_L2_STAR                            |
#---------------------------------------------------------------------------|

discrepancyL2_EP<-function(m,i1,i2,k,p)
{
  n<-nrow(m)
  d<-ncol(m)
  dL2_2<-p+(term_oneDD(i1,m)*alpha_oneDD(i1,i2,k,m))/(n^2)-beta_oneDD(i1,i2,k,m)*term_twoDD(i1,m)*((2^(1-d))/n)-discrepancyTERM_L2_STAR(i1,i1,m)
  dL2_2<-dL2_2+(term_oneDD(i2,m)/(alpha_oneDD(i1,i2,k,m)*(n^2)))-((term_twoDD(i2,m)*((2^(1-d))/n))/(beta_oneDD(i1,i2,k,m)))-discrepancyTERM_L2_STAR(i2,i2,m)
  y<-rep(0,n-2)
  w<-0
  for (j in 1:n)
  {if (j!=i1 & j!=i2)
  {
    w<-w+1
    y[w]<-gamma_oneDD(i1,i2,k,j,m)*discrepancyTERM_L2_STAR(i1,j,m)-discrepancyTERM_L2_STAR(i1,j,m)+(discrepancyTERM_L2_STAR(i2,j,m)/(gamma_oneDD(i1,i2,k,j,m)))-discrepancyTERM_L2_STAR(i2,j,m)
  }
   
  }
  
  y<-sum(y)
  dL2_2<-dL2_2+2*y
  
  return(dL2_2)
}

#####gammaWDD#####

gammaWDD=function(i1,i2,k,j,m)
{
  gamma<-((3/2)-(abs(m[i2,k]-m[j,k])*(1-abs(m[i2,k]-m[j,k]))))/((3/2)-(abs(m[i1,k]-m[j,k])*(1-abs(m[i1,k]-m[j,k]))))
  gamma
}

#####ccWDD#####

ccWDD=function(i,j,m)
{
  n<-nrow(m)
  
  if(i!=j)
  {c<-(3/2)-((abs(m[i,]-m[j,]))*(1-abs(m[i,]-m[j,])))
   c<-(prod(c))/(n^2)}
  c
}


#####discrepancyW2_EP#####
#---------------------------------------------------------------------------|
#args :  m     : the design before the EP                                   |
#        i1,i2 : the two ligns of the lhs elementary permutation            |
#        k     : the column of lhs elementary permutation                   |
#        p     : the square of the W2_star_discrepancy of m                 |
#out     dW2_2 : the square of the W2_star_discrepancy of the new design    |
#depends       : ccWDD, gammaWDD                                            |
#---------------------------------------------------------------------------|

discrepancyW2_EP=function(m,i1,i2,k,p)
{
  # m=le LHS initial, i1 et i2=les deux lignes de la permutation, k la colonne,  
  # DW=discrepance Wrap-around de m au carre
  
  n<-nrow(m)
  dW2_2<-rep(0,n-2)
  w<-0
  for (j in 1:n)
  {if (j!=i1 & j!=i2)
    
  {w<-w+1
   dW2_2[w]<-gammaWDD(i1,i2,k,j,m)*ccWDD(i1,j,m)-ccWDD(i1,j,m)+ccWDD(i2,j,m)/(gammaWDD(i1,i2,k,j,m))-ccWDD(i2,j,m)}
   
  }
  
  dW2_2<-sum(dW2_2)
  dW2_2<-p+2*dW2_2
  return(dW2_2)
  
}

#####discrepancyW2_EP_ESE#####
#---------------------------------------------------------------------------|
#args :  m     : the design before the EP                                   |
#        k     : the column of lhs elementary permutation                   |
#        p     : the square of the W2_star_discrepancy of m                 |
#out     l     : a list with new design and it W2 discrepancy value         |
#depends       : ccWDD, gammaWDD                                            |
#---------------------------------------------------------------------------|

discrepancyW2_EP_ESE=function(m,k,p)    
{
  n<-nrow(m)
  G<-m
  i<-trunc(runif(2,0,n))+1  
  i1=i[1]
  i2=i[2]
  x<-G[i1,k]
  G[i1,k]<-G[i2,k]
  G[i2,k]<-x 
  
  
  
  dW2<-rep(0,n-2)
  w<-0
  for (j in 1:n)
  {if (j!=i1 & j!=i2)
    
  {w<-w+1
   dW2[w]<-gammaWDD(i1,i2,k,j,m)*ccWDD(i1,j,m)-ccWDD(i1,j,m)+ccWDD(i2,j,m)/(gammaWDD(i1,i2,k,j,m))-ccWDD(i2,j,m)}
   
  }
  
  dW2<-sum(dW2)
  dW2<-sqrt(p+2*dW2)
  l<-list(dW2,G)
  return(l)
  
}



#####alphaDD#####

alphaDD<-function(i1,i2,k,m)
{
  
  alpha<-(1+abs(m[i2,k]))/(1+abs(m[i1,k]))
  alpha
}

#####betaDD#####

betaDD<-function(i1,i2,k,m)
{
  
  beta<-(2-abs(m[i2,k]))/(2-abs(m[i1,k]))
  beta
}

#####gammaDD#####

gammaDD<-function(i1,i2,k,j,m)
{
  
  gamma<-(2+abs(m[i2,k])+abs(m[j,k])-abs(m[i2,k]-m[j,k]))/(2+abs(m[i1,k])+abs(m[j,k])-abs(m[i1,k]-m[j,k]))
  gamma
}

#####gDD#####

gDD<-function(i,m)
{
  g<-1+abs(m[i,])
  g<-prod(g)
  g
}

#####hDD#####

hDD<-function(i,m)
{
  h<-1+(abs(m[i,])/2)-0.5*(m[i,]^2)
  h<-prod(h)
  h
}

#####ccDD#####

ccDD<-function(i,j,m)
{
  n<-nrow(m)
  
  if(i!=j)
  {c<-0.5*(2+abs(m[i,])+abs(m[j,])-abs(m[i,]-m[j,]))
   c<-(prod(c))/(n^2)}
  else {
    c<-gDD(i,m)/(n^2)-2*hDD(i,m)/n}
  c
}


#####discrepancyC2_EP#####
#---------------------------------------------------------------------------|
#args :  m     : the design before the EP                                   |
#        i1,i2 : the two ligns of the lhs elementary permutation            |
#        k     : the column of lhs elementary permutation                   |
#        p     : the square of the C2_star_discrepancy of m                 |
#out     dC2_2 : the square of the C2_star_discrepancy of the new design    |
#depends       : alphaDD, betaDD, gammaDD, gDD, hDD, ccDD                   |
#---------------------------------------------------------------------------|

discrepancyC2_EP=function(m,i1,i2,k,p)
{        
# m=le LHS initial, i1 et i2=les deux lignes de la permutation, k la colonne,  
# DW=discrepance Wrap-around de m au carre
  
  n<-nrow(m)
  m<-m-0.5
  dC2_2<-p+(gDD(i1,m)*alphaDD(i1,i2,k,m))/(n^2)-2*alphaDD(i1,i2,k,m)*betaDD(i1,i2,k,m)*hDD(i1,m)/n
  dC2_2<-dC2_2-ccDD(i1,i1,m)-ccDD(i2,i2,m) 
  dC2_2<-dC2_2+(gDD(i2,m)/((n^2)*alphaDD(i1,i2,k,m)))-(2*hDD(i2,m)/(n*alphaDD(i1,i2,k,m)*betaDD(i1,i2,k,m)))
  
  
  x<-rep(0,n-2)
  w<-0
  for (j in 1:n)
  {if (j!=i1 & j!=i2)
  {
    w<-w+1
    x[w]<-gammaDD(i1,i2,k,j,m)*ccDD(i1,j,m)-ccDD(i1,j,m)+ccDD(i2,j,m)/(gammaDD(i1,i2,k,j,m))-ccDD(i2,j,m)}
  } 
  
  x<-sum(x)
  dC2_2<-dC2_2+2*x
  return(dC2_2)
  
  
}


#####discrepancyC2_EP_ESE#####
#---------------------------------------------------------------------------|
#args :  m     : the design before the EP                                   |
#        k     : the column of lhs elementary permutation                   |
#        p     : the square of the C2_star_discrepancy of m                 |
#out     l     : a list with new design and it C2 discrepancy value         |
#depends       : alphaDD, betaDD, gammaDD, gDD, hDD, ccDD                   |
#---------------------------------------------------------------------------|

discrepancyC2_EP_ESE=function(m,k,p)
{
# m=le LHS initial,     i1 et i2=les deux lignes de la permutation, k la colonne,  
# DW=discrepance Wrap-around de m au carre
  n<-nrow(m)
  G<-m
  i<-trunc(runif(2,0,n))+1 
  i1=i[1]
  i2=i[2] 
  x<-G[i1,k]
  G[i1,k]<-G[i2,k]
  G[i2,k]<-x 
  
  m<-m-0.5
  dC2<-p+(gDD(i1,m)*alphaDD(i1,i2,k,m))/(n^2)-2*alphaDD(i1,i2,k,m)*betaDD(i1,i2,k,m)*hDD(i1,m)/n
  dC2<-dC2-ccDD(i1,i1,m)-ccDD(i2,i2,m) 
  dC2<-dC2+(gDD(i2,m)/((n^2)*alphaDD(i1,i2,k,m)))-(2*hDD(i2,m)/(n*alphaDD(i1,i2,k,m)*betaDD(i1,i2,k,m)))
  
  
  x<-rep(0,n-2)
  w<-0
  for (j in 1:n)
  {if (j!=i1 & j!=i2)
  {
    w<-w+1
    x[w]<-gammaDD(i1,i2,k,j,m)*ccDD(i1,j,m)-ccDD(i1,j,m)+ccDD(i2,j,m)/(gammaDD(i1,i2,k,j,m))-ccDD(i2,j,m)}
  } 
  
  x<-sum(x)
  dC2<-sqrt(dC2+2*x)
  l=list(dC2,G)
  return(l)  
}

lhsDesign <- function(n, dimension, randomized=TRUE, seed=NULL){
  
  # arguments : n         = number of points
  #             dimension = number of variables
  #  	    randomized = logical for randomized or centered points
  #             seed       = value of the random seed
  # output : a list containing the arguments and the LHS design
  
  
  # if no seed is provided in argument, choice of the seed for 'runif' and 'sample'
  if (is.null(seed)){
    seed <- as.numeric(Sys.time())
  }
  set.seed(seed)
  
  # Randomized LHS: U[0,1]-sampling of n x dim values
  if (randomized) ran = matrix(runif(n*dimension),nrow=n,ncol=dimension) 
  # Centered LHS
  else ran = matrix(0.5,nrow=n,ncol=dimension) 
    
  x = matrix(0,nrow=n,ncol=dimension)  # initializing matrix x
  
  for (i in 1:dimension) {
    idx = sample(1:n)        # vector of permutations of [1 to n]
    P = (idx-ran[,i]) / n    # vecteur of probabilities
    x[,i] <- P  }
  
  # Outputs:
  return(list(n=n,dimension=dimension,design=x,randomized=randomized,seed=seed))
}

faureprimeDesign <- function(dimension, u = 2, range = c(0, -1)){
	if (!is.null(range)) {
		stopifnot(!any(c(NA, NaN, -Inf, Inf) %in% range))
		stopifnot(length(range) == 2L)
	}
	stopifnot(u >= 2)
	stopifnot(dimension >= 3)
	stopifnot(dimension <= 199)
	u         <- floor(u)
	dimension <- floor(dimension)
	primesupto199 <- c(2,3,5,7,11,13,17,19,23,29,31,37,41,43,47,53,59,61,67,
					   71,73,79,83,89,97,101,103,107,109,113,127,131,137,
					   139,149,151,157,163,167,173,179,181,191,193,197,199)
	prime  <- primesupto199[primesupto199 >= dimension][1]
	stopifnot(prime^(u+1) <= 200^3)
	pu     <- prime^u
	n      <- pu-1
	design <- pu*runif.faure(n, dimension)$design
	if (!is.null(range)) {
		range <- as.numeric(range)
		if (identical(range, c(0,0))   | identical(range, c(0,1)) | 
			identical(range, c(0,-1))  | identical(range, c(1,1)) | 
			identical(range, c(-1,-1)) | identical(range, c(-1,1))) {
			# c(0, 0)  (0, n) already defined
			# c(1,1)   (1-n, n-1) = (2-p^u, p^u -2)
			# c(0,1)   (0, 1)
			# c(0,-1)  (p^-u, 1-p^-u)
			# c(-1,-1) (-1+2p^(-u) , 1-2p^(-u))
			# c(-1, 1) (-1, 1)
			if (identical(range, c(1,1)))   design <- 2*design -n-1  
			if (identical(range, c(0,1)))   design <- (-1 +design)/(n-1)
			if (identical(range, c(0,-1)))  design <- design/(n+1) 
			if (identical(range, c(-1,-1))) design <- (2*design -n-1)/(n+1)
			if (identical(range, c(-1,1)))  design <- (2*design -n-1)/(n-1)
		} else {
			stopifnot(range[2] > range[1])
			design <- (-1 +design)/(n-1)*diff(range) +range[1]
		}
	}
	list(design = design, n = (prime^u)-1, dimension = dimension, 
         prime = prime, u = u)
}

############################################################
# FUNCTIONS OF THIS FILE:
# discrepSA_LHS,discrepESE_LHS: low-discrepancy LHS
# maximinSA_LHS,maximinESE_LHS: Maximin LHS

# Optimization algorithms:
# SA  = Simulated Annealing 
# ESE = Enhanced Stochastic Evolutionary 

#####REFERENCES#####

#Damblin G., Couplet M., and Iooss B. (2013). Numerical studies of space filling designs:
#optimization of Latin Hypercube Samples and subprojection properties, Journal of Simulation, 7:276-289, 2013.

#D.Morris and J.Mitchell. Exploratory designs for computationnal experiments. Journal of 
#Statistical Planning and Inference, 43:381-402, 1995.

#R.JIN, W.CHEN, A.SUDJIANTO. An efficient algorithm for constructing optimal design
#of computer experiments. Journal of Statistical Planning and Inference, 134:268-287, 2005

###############################################################


#####discrepSA_LHS#####
#####L2 DISCREPANCY LHS VIA SIMULATED ANNEALING OPTIMIZATION#####

#---------------------------------------------------------------------------|
#args :  m     : the design                                                 |
#        T0    : the initial temperature                                    |
#        c     : paramater regulating the temperature                       |
#        it    : the number of iterations                                   |
#    criterion : criterion to be optimized ("C2","W2" or "L2star")          |
#      profile : temperature down profile                                   |
#                "GEOM" or "GEOM_MORRIS" or "LINEAR". By default : "GEOM"   |
#       Imax   :parameter adjusting number of iterations without improvement|
#               (if you choose "GEOM_MORRIS" profile)                       |
#output        : a list containing all the input arguments plus:            |
#       low L2_discrepancy design                                           |
#       vector of criterion values along the iterations                     |
#       vector of temperature values along the iterations                   |
#       vector of acceptation probability values along the iterations       |
#depends :  discrepancyCriteria, discrepancyW2_EP, lhs_EP, discrepancyL2_EP |
#           discrepancyC2_EP, lhs_EP                                        |
#---------------------------------------------------------------------------|
   

discrepSA_LHS<-function(design,T0=10,c=0.95,it=2000,criterion="C2",profile="GEOM",Imax=100) 

{ 
  m <- design
  crit <- NULL ; temp <- NULL ; proba <- NULL
  
  if(criterion=="C2")
    
  {if(profile=="GEOM"){
    
    i<-0
    T<-T0
    v<-discrepancyCriteria(m,type='C2')[[1]]  
    crit <- v
    while(T>0 & i<it){
      
      G<-lhs_EP(m)
      g<-discrepancyC2_EP(m,G[[2]],G[[3]],G[[4]],v^2)
      g<-sqrt(g)
      
      diff<-min(exp((v-g)/T),1)
      if (diff == 1){m<-G[[1]]
                     v<-g  
      }
      else         {
        Bernoulli<-rbinom(1,1,diff)
        if (Bernoulli==1){m=G[[1]]
                          v<-g    }}
      i<-i+1
      crit <- c(crit,v) ; temp <- c(temp,T) ; proba <- c(proba,diff)
      T<-(c^i)*(T0)
    }
    
    List.res <- list(design,T0,c,it,criterion,profile,Imax,m,crit,temp,proba)
    names(List.res) <-c("InitialDesign","TO","c","it","criterion","profile","Imax","design","critValues","tempValues","probaValues")
  }
  
  if(profile=="LINEAR"){
  
  i<-0
  T<-T0
  v<-discrepancyCriteria(m,type='C2')[[1]]  
  crit <- v
  while(T>0 & i<it){
    
    G<-lhs_EP(m)
    g<-discrepancyC2_EP(m,G[[2]],G[[3]],G[[4]],v^2)
    g<-sqrt(g)
    
    diff<-min(exp((v-g)/T),1)
    if (diff == 1){m<-G[[1]]
                   v<-g  
    }
    else         {
      Bernoulli<-rbinom(1,1,diff)
      if (Bernoulli==1){m=G[[1]]
                        v<-g    }}
    i<-i+1
    crit <- c(crit,v) ; temp <- c(temp,T) ; proba <- c(proba,diff)
    T<-T0*(1-i/it)
  }
  
  List.res <- list(design,T0,c,it,criterion,profile,Imax,m,crit,temp,proba)
  names(List.res) <-c("InitialDesign","TO","c","it","criterion","profile","Imax","design","critValues","tempValues","probaValues")
}

if(profile=="MC"){
  
  i<-0
  T<-T0
  v<-discrepancyCriteria(m,type='C2')[[1]]  
  crit <- v
  
  n <- dim(design)[1]
  nbvar <- dim(design)[2]
  
  while(T>0 & i<it){
    
#    G<-lhs_EP(m)
    j <- sample(1:n,1)
    new <- runif(nbvar)
    G <- m
    G[j,] <- new
    
#    g<-discrepancyC2_EP(m,G[[2]],G[[3]],G[[4]],v^2)
    g <- discrepancyCriteria(G,type='C2')[[1]] 
    
#    g<-sqrt(g)
    
    diff<-min(exp((v-g)/T),1)
    if (diff == 1){m<-G
                   v<-g  
    }
    else         {
      Bernoulli<-rbinom(1,1,diff)
      if (Bernoulli==1){m=G
                        v<-g    }}
    i<-i+1
    crit <- c(crit,v) ; temp <- c(temp,T) ; proba <- c(proba,diff)
    T<-(c^i)*(T0)
  }
  
  List.res <- list(design,T0,c,it,criterion,profile,Imax,m,crit,temp,proba)
  names(List.res) <-c("InitialDesign","TO","c","it","criterion","profile","Imax","design","critValues","tempValues","probaValues")
}
 
    if(profile=="GEOM_MORRIS"){

       
       T<-T0
       Dbest<-m
       v<-discrepancyCriteria(m,type='C2')[[1]]
       ref<-v
       crit <- v
       for (i in 1:it)
         {
          flag<-0
          I<-1
          while(I<Imax){
                     
             G<-lhs_EP(m)
             g<-discrepancyC2_EP(m,G[[2]],G[[3]],G[[4]],v^2)
             g=sqrt(g)
             diff<-min(exp((v-g)/T),1)
             if (diff == 1){m<-G[[1]]
                         v<-g   
                         flag=1
                       
                       if(v<ref){ref=v
                                  Dbest=m
                                  I=1}
                          else {I=I+1}
                         }
                        
             else {Bernoulli=rbinom(1,1,diff)
                   if (Bernoulli==1){m=G[[1]]
                                    v=g 
                                    flag=1  
                         if (v<ref){ref=v
                                    Dbest=m  
                                    I=1}
                            else {I=I+1}}
                                    
                   else {I=I+1}     
             }
             crit <- c(crit,ref) ; temp <- c(temp,T) ; proba <- c(proba,diff)
         } 
          
       if (flag==1) {T=T*c}
               else {break}
        } 

        List.res <- list(design,T0,c,it,criterion,profile,Imax,Dbest,crit,temp,proba)
        names(List.res) <-c("InitialDesign","TO","c","it","criterion","profile","Imax","design","critValues","tempValues","probaValues")
    }
   } # end if(criterion=="C2")


   
   if(criterion=="L2star")
    {if(profile=="GEOM")
          {i<-0
           T<-T0
           v<-discrepancyCriteria(m,type='L2star')[[1]]	
           crit <- v
           while(T>0 & i<it){
           
                    G<-lhs_EP(m)
                    g<-discrepancyL2_EP(m,G[[2]],G[[3]],G[[4]],v^2)
                    g<-sqrt(g)
                 
                    diff<-min(exp((v-g)/T),1)
                        if (diff == 1){m<-G[[1]]
                                  v<-g    
                                 }
                    else         {
                                 Bernoulli<-rbinom(1,1,diff)
                                 if (Bernoulli==1){m=G[[1]]
                                    v<-g    }}
                  i<-i+1
                  crit <- c(crit,v) ; temp <- c(temp,T) ; proba <- c(proba,diff)
                  T<-(c^i)*(T0)
           }
           List.res <- list(design,T0,c,it,criterion,profile,Imax,m,crit,temp,proba)
           names(List.res) <-c("InitialDesign","TO","c","it","criterion","profile","Imax","design","critValues","tempValues","probaValues")
    }   
    if(profile=="LINEAR")
{i<-0
 T<-T0
 v<-discrepancyCriteria(m,type='L2star')[[1]]  
 crit <- v
 while(T>0 & i<it){
   
   G<-lhs_EP(m)
   g<-discrepancyL2_EP(m,G[[2]],G[[3]],G[[4]],v^2)
   g<-sqrt(g)
   
   diff<-min(exp((v-g)/T),1)
   if (diff == 1){m<-G[[1]]
                  v<-g    
   }
   else         {
     Bernoulli<-rbinom(1,1,diff)
     if (Bernoulli==1){m=G[[1]]
                       v<-g    }}
   i<-i+1
   crit <- c(crit,v) ; temp <- c(temp,T) ; proba <- c(proba,diff)
   T<-T0*(1-i/it)
 }
 List.res <- list(design,T0,c,it,criterion,profile,Imax,m,crit,temp,proba)
 names(List.res) <-c("InitialDesign","TO","c","it","criterion","profile","Imax","design","critValues","tempValues","probaValues")
}   
      if(profile=="GEOM_MORRIS")

       {
        T<-T0
        Dbest<-m
        v<-discrepancyCriteria(m,type='L2star')[[1]]  
        ref<-v
        crit <- ref
        for (i in 1:it)
         {
          flag<-0
          I<-1
          while(I<Imax){
                     
             G<-lhs_EP(m)
             g<-discrepancyL2_EP(m,G[[2]],G[[3]],G[[4]],v^2)
             g=sqrt(g)
             diff<-min(exp((v-g)/T),1)
             if (diff == 1){m<-G[[1]]
                         v<-g   
                         flag=1
                       
                       if(v<ref){ref=v
                                  Dbest=m
                                  I=1}
                          else {I=I+1}
                         }
                        
             else {Bernoulli=rbinom(1,1,diff)
                   if (Bernoulli==1){m=G[[1]]
                                    v=g 
                                    flag=1  
                         if (v<ref){ref=v
                                    Dbest=m  
                                    I=1}
                            else {I=I+1}}
                                    
                   else {I=I+1}     
                 }
             crit <- c(crit,ref) ; temp <- c(temp,T) ; proba <- c(proba,diff)
         } 
          
       if (flag==1) {T=T*c}
               else {break}
        } 
        
        List.res <- list(design,T0,c,it,criterion,profile,Imax,Dbest,crit,temp,proba)
        names(List.res) <-c("InitialDesign","TO","c","it","criterion","profile","Imax","design","critValues","tempValues","probaValues")
      }
   } # end if(criterion=="L2star")
  
   if(criterion=="W2")
    {if(profile=="GEOM")
          {i<-0
           T<-T0
           v<-discrepancyCriteria(m,type='W2')[[1]]	
           crit <- v
           while(T>0 & i<it){
           
                G<-lhs_EP(m)
                g<-discrepancyW2_EP(m,G[[2]],G[[3]],G[[4]],v^2)
                g<-sqrt(g)
                 
                diff<-min(exp((v-g)/T),1)
                      if (diff == 1){m<-G[[1]]
                                  v<-g   
                                 }
                    else         {
                                 Bernoulli<-rbinom(1,1,diff)
                                 if (Bernoulli==1){m=G[[1]]
                                    v<-g    }}
                     i<-i+1
                crit <- c(crit,v) ; temp <- c(temp,T) ; proba <- c(proba,diff)
                T<-(c^i)*(T0)
                    }
           List.res <- list(design,T0,c,it,criterion,profile,Imax,m,crit,temp,proba)
           names(List.res) <-c("InitialDesign","TO","c","it","criterion","profile","Imax","design","critValues","tempValues","probaValues")
           
           }
     if(profile=="LINEAR")
     {i<-0
      T<-T0
      v<-discrepancyCriteria(m,type='W2')[[1]]  
      crit <- v
      while(T>0 & i<it){
        
        G<-lhs_EP(m)
        g<-discrepancyW2_EP(m,G[[2]],G[[3]],G[[4]],v^2)
        g<-sqrt(g)
        
        diff<-min(exp((v-g)/T),1)
        if (diff == 1){m<-G[[1]]
                       v<-g   
        }
        else         {
          Bernoulli<-rbinom(1,1,diff)
          if (Bernoulli==1){m=G[[1]]
                            v<-g    }}
        i<-i+1
        crit <- c(crit,v) ; temp <- c(temp,T) ; proba <- c(proba,diff)
        T<-T0*(1-i/it)
      }
      List.res <- list(design,T0,c,it,criterion,profile,Imax,m,crit,temp,proba)
      names(List.res) <-c("InitialDesign","TO","c","it","criterion","profile","Imax","design","critValues","tempValues","probaValues")
     }
     
       if(profile=="GEOM_MORRIS")

       {
        T<-T0
        Dbest<-m
        v<-discrepancyCriteria(m,type='W2')[[1]]
        ref<-v
        crit <- ref
        for (i in 1:it)
         {
          flag<-0
          I<-1
          while(I<Imax){
                     
             G<-lhs_EP(m)
             g<-discrepancyW2_EP(m,G[[2]],G[[3]],G[[4]],v^2)
             g=sqrt(g)
             diff<-min(exp((v-g)/T),1)
             if (diff == 1){m<-G[[1]]
                         v<-g   
                         flag=1
                       
                       if(v<ref){ref=v
                                  Dbest=m
                                  I=1}
                          else {I=I+1}
                        }
                        
             else {Bernoulli=rbinom(1,1,diff)
                   if (Bernoulli==1){m=G[[1]]
                                    v=g 
                                    flag=1  
                         if (v<ref){ref=v
                                    Dbest=m  
                                    I=1}
                            else {I=I+1}}
                                    
                   else {I=I+1}     
                 }
             crit <- c(crit,ref) ; temp <- c(temp,T) ; proba <- c(proba,diff)
         } 
          
       if (flag==1) {T=T*c}
               else {break}
        } 
        
        List.res <- list(design,T0,c,it,criterion,profile,Imax,Dbest,crit,temp,proba)
        names(List.res) <-c("InitialDesign","TO","c","it","criterion","profile","Imax","design","critValues","tempValues","probaValues")
       }
     } # end if(criterion=="W2")
  
     return(List.res)
} # end function

#####Function to extract elements from lists composed with lists#####
extract_list<-function(l){return(l[[1]])}



#####discrepESE_LHS#####
#####L2 DISCREPANCY LHS VIA ESE OPTIMIZATION#####

#---------------------------------------------------------------------------|
#args :  design     : the design                                            |
#        T0    : the initial temperature                                    |
#        inner_it  : number of iterations for inner loop                    |
#        J     : number of new proposed LHS in inner loop                   |
#        it    : number of iterations for outer loop                        |
#        criterion: "C2", "W2" or "L2star"                               |
#output        : a list containing all the input arguments plus:            |
#       low L2_discrepancy design                                           |
#       vector of criterion values along the iterations                     |
#       vector of temperature values along the iterations                   |
#       vector of acceptation probability values along the iterations       |
#depends :  discrepancyL2_EP_ESE, discrepancyW2_EP_ESE, discrepancyL2_STAR  |
#           discrepancyC2_EP_ESE, discrepancyCriteria                       |
#---------------------------------------------------------------------------|


discrepESE_LHS<-function(design,T0=0.005*discrepancyCriteria(design,type='C2')[[1]],inner_it=100,J=50,it=2,criterion="C2")
{
  m<-design
  crit <- NULL ; temp <- NULL ; proba <- NULL
  
  if(criterion=="C2")
  {
  d<-ncol(m)
  Temperature<-T0
  Best<-m
  dC2<-discrepancyCriteria(m,type='C2')[[1]]                   
  best<-dC2
  crit <- dC2

  for (q in 1:it)
   {
     
     BOLD<-Best
     bold<-best                                   # Best=new LHS built at every step        
                                                # BOLD= new LHS built at each iteration q
     ni<-0
     count<-0
     na<-0
     while(count<=inner_it)                      # inner loop
     {
       count<-count+1
   

     modulo<-count%%d                         # d : number of columns of m
     l<-list(m)
     l<-rep(l,J) #liste de liste
     
     g<-lapply(l,discrepancyC2_EP_ESE,k=modulo+1,p=discrepancyCriteria(m,type='C2')[[1]]^2)
     values<-lapply(g,extract_list) 
     k<-which.min(values)
     a<-values[[k]]  
                                             
       Delta<-a-dC2
                                                
     if((Delta)<=(Temperature*runif(1)))                 # higher is the temperature, higher is the probability of accepting a bad design.
                                                # if Delta is low, the probability is high of accepting a bad design.   
                                                # if Delta>Temperature, m is never accept.
       
     
          {m<-g[[k]][[2]]
           
           dC2<-a
                    
          na<-na+1
             if(a<=best)  
                 {Best<-m
                  best<-a
                 ni<-ni+1}                       #if optimization is ok, ni=ni+1
           }
       crit <- c(crit,best)
    }
          
  v1<-na/inner_it    # v1<-acceptance ratio
  v2<-ni/inner_it    # v2<-optimization ratio
  
  temp <- c(temp,rep(Temperature,inner_it)) ; proba <- c(proba,rep(v1,inner_it))   

  if (best-bold<0){f<-1
              if(v1>=0.1 & v2<=v1)
                  {Temperature<-0.8*Temperature}
                       else {if (v1>=0.1 & v2==v1){} 
                                      else {Temperature<-Temperature/0.8}
                            }  
                               }

                                               # if the criteria is optimized after the inner loop, then f equals 1
      else {f<-0
      if (v1<=0.1){Temperature<-Temperature/0.7}
              else {Temperature<-Temperature*0.9}
     }

                                               # else, it is the exploratory step

    }
  List.res <- list(design,T0,inner_it,J,it,criterion,Best,crit,temp,proba)
  names(List.res) <-c("InitialDesign","TO","inner_it","J","it","criterion","design","critValues","tempValues","probaValues") 
  
  }


  if(criterion=="L2star")
  {
  d<-ncol(m)
  Temperature<-T0
  Best<-m
  dL2<-discrepancyCriteria(m,type='L2star')[[1]]              
  best<-dL2              

  for (q in 1:it)
   {

     BOLD<-Best 
     bold<-best                                   # Best=new LHS built at every step        
                                                # BOLD= new LHS built at each iteration q
     ni<-0
     count<-0
     na<-0
     while(count<=inner_it)                      # inner loop
     {
       count<-count+1
   

     modulo<-count%%d                         # d : number of columns of m
     l<-list(m)
     l<-rep(l,J) #liste de liste
     
     g<-lapply(l,discrepancyL2_EP_ESE,k=modulo+1,p=dL2^2)
     values<-lapply(g,extract_list) 
     k<-which.min(values)
     a<-values[[k]]
                          
                                               
       Delta<-a-dL2
                                                
     if((Delta)<=(Temperature*runif(1)))                 # higher is the temperature, higher is the probability of accepting a bad design.
                                                # if Delta is low, the probability is high of accepting a bad design.   
                                                # if Delta>Temperature, m is never accept.
       
     
          {m<-g[[k]][[2]]
           
           dL2<-a
                    
          na<-na+1
             if(a<=best)  
                 {Best<-m
                  best<-a
                 ni<-ni+1}                       #if optimization is ok, ni=ni+1
           }
       crit <- c(crit,best)
    }
          
  v1<-na/inner_it    # v1<-acceptance ratio
  v2<-ni/inner_it    # v2<-optimization ratio

  temp <- c(temp,rep(Temperature,inner_it)) ; proba <- c(proba,rep(v1,inner_it))

  if (best-bold<0){f<-1
              if(v1>=0.1 & v2<=v1)
                  {Temperature<-0.8*Temperature}
                       else {if (v1>=0.1 & v2==v1){} 
                                      else {Temperature<-Temperature/0.8}
                            }  
                               }

                                               # if the criteria is optimized after the inner loop, then f equals 1
      else {f<-0
      if (v1<=0.1){Temperature<-Temperature/0.7}
              else {Temperature<-Temperature*0.9}
     }

                                               # else, it is the exploratory step



    }
  List.res <- list(design,T0,inner_it,J,it,criterion,Best,crit,temp,proba)
  names(List.res) <-c("InitialDesign","TO","inner_it","J","it","criterion","design","critValues","tempValues","probaValues") 
  
  }

 if(criterion=="W2")
  {
  d<-ncol(m)
  Temperature<-T0
  Best<-m
  dW2<-discrepancyCriteria(m,type='W2')[[1]]                  
  best<-dW2              

  for (q in 1:it)
   {

     BOLD<-Best
     bold<-best                                    # Best=new LHS built at every step        
                                                # BOLD= new LHS built at each iteration q
     ni<-0
     count<-0
     na<-0
     while(count<=inner_it)                      # inner loop
     {
       count<-count+1
   

     modulo<-count%%d                         # d : number of columns of m
     l<-list(m)
     l<-rep(l,J) #liste de liste
     
     g<-lapply(l,discrepancyW2_EP_ESE,k=modulo+1,p=discrepancyCriteria(m,type='W2')[[1]]^2)
     values<-lapply(g,extract_list) 
     k<-which.min(values)
     a<-values[[k]]
                          
                                               
       Delta<-a-dW2
                                                
     if((Delta)<=(Temperature*runif(1)))                 # higher is the temperature, higher is the probability of accepting a bad design.
                                                # if Delta is low, the probability is high of accepting a bad design.   
                                               # if Delta>Temperature, m is never accept.
       
     
          {m<-g[[k]][[2]]
           
           dW2<-a
                    
          na<-na+1
             if(a<=best)  
                 {Best<-m
                  best<-a
                 ni<-ni+1}                       #if optimization is ok, ni=ni+1
           }
       crit <- c(crit,best)
    }
          
  v1<-na/inner_it    # v1<-acceptance ratio
  v2<-ni/inner_it    # v2<-optimization ratio

  temp <- c(temp,rep(Temperature,inner_it)) ; proba <- c(proba,rep(v1,inner_it))
     
  if (best-bold<0){f<-1
              if(v1>=0.1 & v2<=v1)
                  {Temperature<-0.8*Temperature}
                       else {if (v1>=0.1 & v2==v1){} 
                                      else {Temperature<-Temperature/0.8}
                            }  
                               }

                                               # if the criteria is optimized after the inner loop, then f equals 1
      else {f<-0
      if (v1<=0.1){Temperature<-Temperature/0.7}
              else {Temperature<-Temperature*0.9}
     }

                                               # else, it is the exploratory step

    }
  List.res <- list(design,T0,inner_it,J,it,criterion,Best,crit,temp,proba)
  names(List.res) <-c("InitialDesign","TO","inner_it","J","it","criterion","design","critValues","tempValues","probaValues") 
  
 }

return(List.res)

}


#####maximinSA_LHS#####
#####Maximin LHS VIA SIMULATED ANNEALING OPTIMIZATION#####

#---------------------------------------------------------------------------|
#args :  m     : the design                                                 |
#        T0    : the initial temperature                                    |
#        c     : parameter regulating the temperature                       |
#        it    : the number of iterations                                   |
#        p     : power required in phiP criterion                           |
#      profile : temperature down profile                                   |
#                "GEOM" or "GEOM_MORRIS" or "LINEAR". By default : "GEOM"   |
#output        : a list containing all the input arguments plus:            |
#       a mindist optimized design                                          |
#       vector of criterion values along the iterations                     |
#       vector of temperature values along the iterations                   |
#       vector of acceptation probability values along the iterations       |
#depends :  phiP,lhs_EP                                                     |
#---------------------------------------------------------------------------|
   

maximinSA_LHS<-function(design,T0=10,c=0.95,it=2000,p=50,profile="GEOM",Imax=100) 
  { 
  crit <- NULL ; temp <- NULL ; proba <- NULL
  
  if(profile=="GEOM"){
  m<-design
  i<-0
  T<-T0
  fi_p<-phiP(m,p)
  crit <- fi_p
  
  while(T>0 & i<it)

         {           
               G<-lhs_EP(m)[[1]]
               fi_p_ep<-phiP(G,p)
               diff<-min(exp((fi_p-fi_p_ep)/T),1)
               if (diff == 1){m<-G
                       fi_p<-fi_p_ep
                      }
               else { Bernoulli<-rbinom(1,1,diff)
                      if (Bernoulli==1){m<-G
                                    fi_p<-fi_p_ep    }}

               i<-i+1
               crit <- c(crit,fi_p) ; temp <- c(temp,T) ; proba <- c(proba,diff)
               T<-(c^i)*(T0)
           }
  
  List.res <- list(design,T0,c,it,p,profile,Imax,m,crit,temp,proba)
  names(List.res) <-c("InitialDesign","TO","c","it","p","profile","Imax","design","critValues","tempValues","probaValues") 
  }
  
if(profile=="LINEAR"){
    m<-design
    i<-0
    T<-T0
    fi_p<-phiP(m,p)
    crit <- fi_p
    
    while(T>0 & i<it)
      
    {           
      G<-lhs_EP(m)[[1]]
      fi_p_ep<-phiP(G,p)
      diff<-min(exp((fi_p-fi_p_ep)/T),1)
      if (diff == 1){m<-G
                     fi_p<-fi_p_ep
      }
      else { Bernoulli<-rbinom(1,1,diff)
             if (Bernoulli==1){m<-G
                               fi_p<-fi_p_ep    }}
      
      i<-i+1
      crit <- c(crit,fi_p) ; temp <- c(temp,T) ; proba <- c(proba,diff)
      T<-T0*(1-i/it)
    }
    
    List.res <- list(design,T0,c,it,p,profile,Imax,m,crit,temp,proba)
    names(List.res) <-c("InitialDesign","TO","c","it","p","profile","Imax","design","critValues","tempValues","probaValues") 
  }
  
  if(profile=="GEOM_MORRIS") 
     
  {m<-design
   T<-T0
   Dbest<-m
   fi_p=phiP(m,p)
   ref<-fi_p
   crit <- ref
   for (i in 1:it)
   {
     flag=0
     I=1
     while(I<Imax){
                     
           G=lhs_EP(m)[[1]]
           fi_p_ep=phiP(G,p)
           diff=min(exp((fi_p-fi_p_ep)/T),1)
           if (diff == 1){m=G
                       fi_p=fi_p_ep    
                       flag=1
                       
                       if(fi_p<ref){ref=fi_p
                                  Dbest=m
                                  I=1}
                          else {I=I+1}
                       }
                        
           else {Bernoulli=rbinom(1,1,diff)
                   if (Bernoulli==1){m=G
                                    fi_p=fi_p_ep 
                                    flag=1  
                       if (fi_p<ref){ref=fi_p
                                    Dbest=m  
                                    I=1}
                            else {I=I+1}}
                                    
                   else {I=I+1}     
                 }
           crit <- c(crit,ref) ; temp <- c(temp,T) ; proba <- c(proba,diff)
         } 
          
       if (flag==1) {T=T*c}
               else {break}
   } 
   
   List.res <- list(design,T0,c,it,p,profile,Imax,Dbest,crit,temp,proba)
   names(List.res) <-c("InitialDesign","TO","c","it","p","profile","Imax","design","critValues","tempValues","probaValues") 
   
   }
  
  
  if(profile=="MC"){
    m<-design
    i<-0
    T<-T0
    fi_p<-phiP(m,p)
    crit <- fi_p
    n <- dim(design)[1]
    nbvar <- dim(design)[2]
    
    while(T>0 & i<it)
      
    {           
      
#      G<-lhs_EP(m)[[1]]
      j <- sample(1:n,1)
      new <- runif(nbvar)
      G <- m
      G[j,] <- new
      fi_p_ep<-phiP(G,p)
      diff<-min(exp((fi_p-fi_p_ep)/T),1)
      if (diff == 1){m<-G
                     fi_p<-fi_p_ep
      }
      else { Bernoulli<-rbinom(1,1,diff)
             if (Bernoulli==1){m<-G
                               fi_p<-fi_p_ep    }}
      
      i<-i+1
      crit <- c(crit,fi_p) ; temp <- c(temp,T) ; proba <- c(proba,diff)
      T<-(c^i)*(T0)
    }
    
    List.res <- list(design,T0,c,it,p,profile,Imax,m,crit,temp,proba)
    names(List.res) <-c("InitialDesign","TO","c","it","p","profile","Imax","design","critValues","tempValues","probaValues") 
  }
  
  
   return(List.res)
}

#####maximinESE_LHS#####
#####Maximin LHS VIA ESE OPTIMIZATION#####

#---------------------------------------------------------------------------|
#args :  design: the design                                                 |
#        T0    : the initial temperature                                    |
#        inner_it  : number of iterations for inner loop                    |
#        J     : number of new proposed LHS in inner loop                   |
#        it    : number of iterations for outer loop                        |
#        p     : the power in phiP criterion                                |
#output        : a list containing all the input arguments plus:            |
#       a mindist optimized design                                          |
#       vector of criterion values along the iterations                     |
#       vector of temperature values along the iterations                   |
#       vector of acceptation probability values along the iterations       |
#depends :     phiP_EP_ESE, phiP                                            |
#---------------------------------------------------------------------------|

maximinESE_LHS<-function(design,T0=0.005*phiP(design,p=50),inner_it=100,J=50,it=1,p=50)
{
  m<-design
  crit <- NULL ; temp <- NULL ; proba <- NULL
  
  d<-ncol(m)
  Temperature<-T0
  Best<-m
  fi_p<-phiP(m,p)                   
  best<-fi_p
  crit <- fi_p

  for (q in 1:it)
   {
     BOLD<-Best
     bold<-best                              # B=new LHS built at every step        
                                             # BOLD= new LHS built at each iteration q
     ni<-0
     count<-0
     na<-0
     while(count<=inner_it)                      # inner loop
     {
       count<-count+1
   

     modulo<-count%%d                         # d : number of columns of m
     l<-list(m)
     l<-rep(l,J) 
     
     g<-lapply(l,phiP_EP_ESE,k=modulo+1,p=p)
     values<-lapply(g,extract_list) 
     k<-which.min(values)
     a<-values[[k]]
                                               
       Delta<-a-fi_p
                                                
     if((Delta)<=(Temperature*runif(1)))        # higher is the temperature, higher is the probability of accepting a bad design.
                                                # if Delta is low, the probability is high of accepting a bad design.   
                                                # if Delta>Temperature, m is never accept.
       
     
          {m<-g[[k]][[2]]
           
           fi_p<-a
                    
          na<-na+1
             if(a<=best)  
                 {Best<-m
                  best<-a
                 ni<-ni+1}                       #if optimization is ok, ni=ni+1
           }
       crit <- c(crit,best)
    }
          
  v1<-na/inner_it    # v1<-acceptance ratio
  v2<-ni/inner_it    # v2<-optimization ratio
     
  temp <- c(temp,rep(Temperature,inner_it)) ; proba <- c(proba,rep(v1,inner_it))

  if (best-bold<0){f<-1
              if(v1>=0.1 & v2<=v1)
                  {Temperature<-0.8*Temperature}
                       else {if (v1>=0.1 & v2==v1){} 
                                      else {Temperature<-Temperature/0.8}
                            }  
                               }

                                               # if the criteria is optimized after the inner loop, then f equals 1
      else {f<-0
      if (v1<=0.1){Temperature<-Temperature/0.7}
              else {Temperature<-Temperature*0.9}
     }
}
  
List.res <- list(design,T0,inner_it,J,it,p,Best,crit,temp,proba)
names(List.res) <-c("InitialDesign","TO","inner_it","J","it","p","design","critValues","tempValues","probaValues") 
     # else, it is the exploratory step

return(List.res)
}

#####phiP#####

# Compute the phiP criterion 
# (Lp norm of the sum of the inverses of the design inter-point distances)
# Reference: Pronzato, L. and Muller, W.,2012, Design of computer experiments: 
#              space filling and beyond, Statistics and Computing, 22:681-701.
# A higher phiP corresponds to a more regular scaterring of design points

#---------------------------------------------------------------------------|
#args :  design     : the design                                            |
#        p          : the "p" in the Lp norm which is taken (default=50)    |
#out  : the phiP criterion                                                  |
#---------------------------------------------------------------------------|

phiP<-function(design,p=50)   
{
  D<-dist(design)
  D<-D^(-p)
  fi_p<-(sum(D))^(1/p)
  return(fi_p)  
}

nolhDesign <- function(dimension, range = c(0, 1)) {
    # NOLHdes <- get(load(system.file("data", "NOLHdesigns.rda", package = "DiceDesign")))
    NOLHdes   <- get("NOLHdesigns", pos = "package:DiceDesign")
    dfr <-  if (dimension %in% 2:7) NOLHdes[["nolh2_7"]]
                else if (dimension %in% 8:11) NOLHdes[["nolh8_11"]]
                    else if (dimension %in% 12:16) NOLHdes[["nolh12_16"]]
                        else if (dimension %in% 17:22) NOLHdes[["nolh17_22"]]
                            else if (dimension %in% 23:29) NOLHdes[["nolh23_29"]]
                                else stop("dimension must be in 2:29")
    # dfr <-  if (dimension %in% 2:7) NOLHdesigns[["nolh2_7"]]
                # else if (dimension %in% 8:11) NOLHdesigns[["nolh8_11"]]
                    # else if (dimension %in% 12:16) NOLHdesigns[["nolh12_16"]]
                        # else if (dimension %in% 17:22) NOLHdesigns[["nolh17_22"]]
                            # else if (dimension %in% 23:29) NOLHdesigns[["nolh23_29"]]
                                # else stop("dimension must be in 2:29")
    mat <- as.matrix(dfr[,seq_len(dimension)])
    mat <- if (identical(range, c(1,1))) mat 
              else if (identical(range, c(0, 0))) mat +(nrow(mat)-1)/2  
                   else (mat/(nrow(mat)-1) +0.5) *diff(range) + range[1]
    dimnames(mat) <- NULL
    # dimnames(mat) <- list(seq_len(nrow(mat)), 
                          # paste0("X", seq_len(ncol(mat))))
list(n = nrow(mat), dimension = dimension, design = mat)
}

factDesign <- function (dimension, levels) {
    if (length(levels) == 1) {
        levels <- rep(levels, dimension)
    }
    else if (length(levels) != dimension) {
        stop("argument 'levels' must be of length 1 or 'dimension'")
    }
    x <- matrix(0, prod(levels), dimension)
    for (i in 1:dimension) {
        xi <- 1
        for (j in (1:dimension)[(1:dimension) < i]) {
            xi <- xi %x% rep(1, levels[j])
        }
        xi <- xi %x% seq(0, 1, length.out = levels[i])
        for (j in (1:dimension)[(1:dimension) > i]) {
            xi <- xi %x% rep(1, levels[j])
        }
        x[, i] <- xi
    }
    n <- dim(x)[1]
return(list(design = x, n = n, dimension = dimension, levels = levels))
}

meshRatio <- function(design){
	#----------------------------------
	# Compute the meshratio criterion
	# For a regular mesh, ratio=1
	# input : design of n experiments
	# Example : Meshratio(matrix(runif(40),20,2))
	#----------------------------------

	X <- as.matrix(design)
	n <- dim(X)[1]
	dimension <- dim(X)[2]

	if ( n < dimension ){
    		stop('Warning : the number of points is lower than the dimension')
	}

	# To check the experimental region
	if ( min(X)<0 || max(X)>1 ){
		warning("The design is rescaling into the unit cube [0,1]^d.")
		M <- apply(X,2,max)
		m <- apply(X,2,min)
		for (j in 1:dim(X)[2]){
			X[,j] <- (X[,j]-m[j])/(M[j]-m[j])
		}	
	}
	
   	DistanceMax  <- -1.0E30
   	DistanceMin  <-  1.0E30
  
   	for (i in 1:(n-1)) {
      	DistMin  <- 1.0E30
      	DistMax  <- -1.0E30
      	for (k in 1 : n){
           		if (i != k){   # if the differs from current point
				Dist <- 0
				for (j in 1 : dimension){
					Dist  <- Dist + (X[i,j] -X[k,j])*(X[i,j] - X[k,j])
               		}
				if (Dist > DistMax){ DistMax <- Dist; }
					if (Dist < DistMin){ DistMin <- Dist; }
          			}
      		}
     			if (DistanceMax < DistMin){
				DistanceMax <- DistMin
			}
     			if (DistanceMin > DistMin){
			DistanceMin <- DistMin
			}  
   		}
	ratio <- sqrt(DistanceMax/DistanceMin)
   	return(ratio)
}

xDRDN <- function(obj, width = 1, letter = "X", dgts = NULL, range = NULL) {
    mat <-  if (is.list(obj) & "design" %in% names(obj)) {
                obj$design 
            } else { 
                if (is.matrix(obj) | is.data.frame(obj)) { 
                    obj 
                } else {
                    stop("obj must be a matrix, a data.frame or a list with a 'design' field") 
                }
            }
    style <- paste0("%", width, ".", width, "d")
    LET   <- c(LETTERS[c(1:8, 10:26)], letters[c(1:8, 10:26)])
    cn    <- if(width == 0 & ncol(mat) < 51) {
                   LET[seq_len(ncol(mat))]
             } else { 
                    paste0(letter, sprintf(style, seq_len(ncol(mat)))) 
             }
    dimnames(mat) <- list(seq_len(nrow(mat)), cn)
    currange <- range(mat)
    if (!is.null(range)) mat <- (mat - currange[1])/diff(currange)*diff(range) + range[1]
    if (!is.null(dgts))  mat <- round(mat, dgts)
return(mat)
}

#####ALGORITHM of WOOTON SERGENT PHAN-TAN-LUU (WSP)#####


####wspDesign#####

#---------------------------------------------------------------------------|
#args :  design     : matrix (n,d) containing n design points               |
#                     d is the dimension                                    |
#        dmin       : minimal bound for mindist final design                |
#        init       : define the initialization point of the algorithm      |
#out :                a list containing the input arguments plus            |
#                   the space filling design according to mindist criterion |
#---------------------------------------------------------------------------|


wspDesign <- function(design, dmin, init = "center"){  #  dmin must be given

  m <- design
  n <-nrow(m)                     # Number of points in m.
  d <-ncol(m)                     # Dimension of m.
  if (init == "center") A <- rbind(m, rep(0.5, d)) # add center point
  else A <-  rbind(m, runif(d))
  D<-as.matrix(dist(A))
  diag(D)<-rep(Inf,n+1)            # Matrix of distances between all pairs of point (both design points and center)

  base<-as.numeric(which.min(D[n+1,1:n])) # the nearest design point to center point
  D=D[1:n,-(n+1)] 
                                 # Matrix of distances between all pairs of design points
 
  


  points<-base                    # vectors containing base points and suppressed points as well
  x<-base                         # vector containing only base points
  while (length(points)<n){      
              
              co<-as.numeric(which(D[base,]<dmin))
              if(length(co)>0)
               {D[,co]<-rep(Inf,n)            
                D[co,]<-rep(Inf,n)
                points<-c(points,co)}   #suppressed points
                    
                
              new_base<-which.min(D[base,])  #new base point
              points<-c(points,base)
              D[base,]<-D[,base]<-rep(Inf,n)
              base<-new_base
              x<-c(x,base)}
              
   Mres <- m[x,]  # Final matrix    #final space-filling design
   Mresidual <- m[-x, ] # Design of "residual" points (not in Mres)
  
   return(list(InitialDesign = design, dmin = dmin, design = Mres, ResidualDesign = Mresidual))
   
   return(Mres) 
}

	straussDesign <- function(n,dimension,RND,alpha=0.5,repulsion=0.001,NMC=1000,constraints1D=0,repulsion1D=0.0001,seed=NULL){
			
    # if no seed is provided in argument, choice of the seed for 'runif'
	if (is.null(seed)){
		seed <- as.numeric(Sys.time())
	}
	set.seed(seed)	
	
	# Case with potential different to zero
	# the C code is computed for beta = -ln(gamma)
	if (alpha != 0){
		repulsion <- -log(repulsion)
	} # End of the case with no constraint
	
	# Case with no constraints
	if(constraints1D!=0){
		R1D <- 0.75/n 
	}

	# initial design
	init <- runif(n*dimension)
		
 	out <- .C("C_StraussDesign", as.double(init), as.integer(n), 
	as.integer(dimension), as.integer(constraints1D), as.integer(NMC), 
	as.double(RND), as.double(alpha), as.double(repulsion), 
	as.double(repulsion1D),as.integer(seed),ans = double(n * dimension), 
	PACKAGE="DiceDesign")
		
	if (alpha != 0){
		repulsion <- exp(-repulsion)
	}
		
	# Outputs
	if (constraints1D==0){
		return(list(n=n, dimension=dimension, 
			design_init=t(matrix(init,ncol=n,nrow=dimension,byrow=FALSE)), 
			radius=RND, alpha=alpha, repulsion=repulsion, NMC=NMC,    
			seed=seed, constraints1D=constraints1D,
			design=matrix(out$ans,nrow=n,ncol=dimension,byrow=TRUE)))
	} else {
		return(list(n=n,dimension=dimension,
			design_init=matrix(init,ncol=dimension,nrow=n,byrow=TRUE), 	
			radius=RND, alpha=alpha, repulsion=repulsion,NMC=NMC,  	
			seed=seed, constraints1D=constraints1D, repulsion1D=repulsion1D, 
			design=matrix(out$ans,nrow=n, ncol=dimension,byrow=TRUE)))
	}

}

olhDesign <- function(dimension, range = c(0, 1)) {
    funM <- function(dimpow2) {
        if (dimpow2 == 1) { matrix(c(1,2,2,1), 2, 2)
        } else {
            M <- funM(dimpow2 - 1)
            rbind(cbind(M, M+ncol(M)), cbind(M+ncol(M), M))
        }
    }
    funS <- function(dimpow2) {
        if (dimpow2 == 1) { matrix(c(1,1,1,-1), 2, 2)
        } else {
            S <- funS(dimpow2 - 1)
            P <- S[seq(1, nrow(S)/2),, drop = FALSE]
            Q <- S[seq(1+nrow(S)/2, nrow(S)),, drop = FALSE]
            rbind(cbind(P,P), cbind(Q,-Q), cbind(P,-P), cbind(Q,Q))
        }
    }
    funOLH <- function(dimpow2) {
        T <- funM(dimpow2) * funS(dimpow2)
        rbind(T, rep(0, ncol(T)), -T)
    }
    
    dimpow2 <- max(1, ceiling(log2(dimension)))
    mat <- funOLH(dimpow2)[, seq_len(dimension), drop = FALSE]
    mat <- if (identical(range, c(1,1))) mat
              else if (identical(range, c(0, 0))) mat +(nrow(mat)-1)/2  
                   else (mat/(nrow(mat)-1) +0.5) * diff(range) + range[1]
    dimnames(mat) <- NULL
    # dimnames(mat) <- list(seq_len(nrow(mat)), 
                          # paste0("X", seq_len(ncol(mat))))
list(n = nrow(mat), dimension = dimension, design = mat)
}

#####MINIMAL SPANNING TREE (ALGORITHM of PRIM(1957))#####

####mstCriteria#####

#---------------------------------------------------------------------------|
#args :  design     : the design                                            |
#        plot2d     : default="FALSE",  show 2D projections of the mst      |
#out     l     : list with edges and both mean and standard value of the    |
#                lengthes                                                   |
#---------------------------------------------------------------------------|



mstCriteria <- function(design,plot2d="FALSE")

{
  m <- design
  n<-nrow(m)             #number of points 
  D<-as.matrix(dist(m))  #distances matrix two points two
  diag(D)<-rep(Inf,n)    
  x<-rep(0,n-1)          #Initialization of the vector such that each coordinate is an edge length 
  Peak<-1                #First peak of the tree
  p<-Peak
  Tree<-list(c())
  Tree<-rep(Tree,n)      #Initialization of a void list
 
  
#-----------------------------------------------------------|
  if(plot2d)
  {plot(m,pch=19,col="blue",xlab="",ylab="")}        
#-----------------------------------------------------------|


  for( i in 1:(n-1))               
                     
    { 

       if(length(Peak)==1)
              {
                 new_p<-which.min(as.numeric(D[p,]))
                 x[i]<-D[p,new_p]
                 D[new_p,Peak]=D[Peak,new_p]<-Inf
                 Peak<-c(Peak,new_p)
               }


       else{
                 A<-D[Peak,]
                 w<-which.min(A)
                 reste<-w%%nrow(A)
                 if(reste!=0) 

                    {p<-Peak[reste]
                    new_p<-w %/% nrow(A)+1}

               else
                    {p<-Peak[nrow(A)]
                    new_p<-w %/% nrow(A)}
       
       x[i]<-D[p,new_p]
       D[new_p,Peak]<-D[Peak,new_p]<-Inf
       Peak<-c(Peak,new_p)
 
            }    

       Tree[[p]]<-c(Tree[[p]],new_p)
       Tree[[new_p]]<-c(Tree[[new_p]],p)                             
       if (plot2d=="TRUE")
          {lines(rbind(m[p,],m[new_p,]),col='red')}
    }
       return(list(tree=Tree,stats=c(mean(x),sqrt(var(x)))))
}

unscaleDesign <- function(design, min=NULL, max=NULL, uniformize=FALSE, InitialDesign=NULL){
  d <- dim(design)[[2]] ; n <- dim(design)[[1]]
  
  if (uniformize & !is.null(InitialDesign)){ # Inverse Rosenblatt transformation from U[0,1]^d
    for (i in 1:d) x[,i] <- quantile(InitialDesign[,i],design[,i])
  }else x <- design * matrix((max - min), ncol=d,nrow=n, byrow=T) + matrix(min, ncol=d, nrow=n,byrow=T)
  
  return(list(design = x, min = min, max = max, uniformize = uniformize))
}

scaleDesign <- function(design, min=NULL, max=NULL, uniformize=FALSE){
  d <- dim(design)[[2]] ; n <- dim(design)[[1]]
  
  if (uniformize){ # Rosenblatt transformation to U[0,1]^d
    x <- design
    for (i in 1:d) x[,i] <- ecdf(design[,i])(design[,i])
    
  }else{ # scaling between min and max
    if (is.null(min)) min <- apply(design, 2, min)
    if (is.null(max)) max <- apply(design, 2, max)
    x <- ( design - matrix(min, ncol=d, nrow=n, byrow=T) ) / matrix((max - min), ncol=d,nrow=n, byrow=T)
  }
  return(list(design = x, min = min, max = max, uniformize = uniformize, InitialDesign=design))
}

coverage <- function(design){
	# Compute the coverage measure
	# For a regular mesh, c=0
	# input : design of n experiments

	X <- as.matrix(design)
	n <- dim(X)[1]
	dimension <- dim(X)[2]

	if ( n < dimension ){
    		stop('Warning : the number of points is lower than the dimension.')
	}
	# To check the experimental region
	if ( min(X)<0 || max(X)>1 ){
		warning("The design is rescaling into the unit cube [0,1]^d.")
		M <- apply(X,2,max)
		m <- apply(X,2,min)
		for (j in 1:dim(X)[2]){
			X[,j] <- (X[,j]-m[j])/(M[j]-m[j])
		}	
	}
	
	# Compute the distance between the points.
	Distance <- as.matrix(dist(X))
	diag(Distance) <- 10e3
	Dmin <- as.matrix(apply(Distance,2,min))
	gammabar <-(1/n)*sum(Dmin)
	s <- 0

	for (i in 1:n){
    		s <- s+ (Dmin[i]-gammabar)*(Dmin[i]-gammabar)
	}
	cov <-(1/gammabar)*((1/n)*s)^(1/2)
	return(cov)
}

dmaxDesign <- function(n,dimension,range,niter_max=1000,seed=NULL){
	# if no seed is provided in argument, choice of the seed for 'runif'
	if (is.null(seed)){
		seed <- as.numeric(Sys.time())
	}
	set.seed(seed)
	
	# The spherical variogram model
	variosphe <- function(h,a){
		if (h <= a){
			gamma <- 1.5*(h/a)- 0.5*(h/a)^3
		}
		else{
			gamma <- 1
		}
		return(gamma)
	}

	p <- matrix(runif(n*dimension,0,1),n,dimension)
	p_init <- p
	
	# Hypothesis validation
	if (n < dimension){
		stop('Warning : the number of points is lower than the dimension.')
	}

 	if (range == 0){
		# To choose a large range in order to fill the region experimental
		range <- sqrt(dimension)/2 
 	}

	Distance <- as.matrix(dist(p))

	# Compute Correlation Matrix
	C <- diag(1,n,n)
	for (i in 1:(n-1)){
		for (k in (i+1):n){
			if (Distance[i,k] <= range){	 
		 		C[i,k] <- 1-variosphe(Distance[i,k],range)
		 	}
			C[k,i] <- C[i,k]
		}
	}

	# Compute determinant of the matrix C
 	dinit <- det(C)
 
	compt <- 1
	Deter <- rep(0,niter_max)
	Deter[compt] <- dinit

	while(compt < niter_max){
		compt <- compt+1
		# Choose an index 
		iunif <- floor(runif(1,1,n+1))
 		# Choose a potential new point 
		u <- runif(dimension)
		pu <- p
		pu[iunif,] <- u
		# Compute the potential new determinant of the matrix C (compute only row number iunif)
		Distu <- matrix(0,n,n)
       	for (ii in 1:n){
            	for (j in 1:dimension){          
            		Distu[iunif,ii]  <- Distu[iunif,ii] + (pu[iunif,j] - pu[ii,j])*(pu[iunif,j] - pu[ii,j])
               	}		   
       	}	
		Distu[iunif,] <- sqrt(Distu[iunif,])
		Cu <- C
		for (k in 1:n){
			if (k != iunif){
				if (Distu[iunif,k] <= range){	
					Cu[iunif,k] <- 1-variosphe(Distu[iunif,k],range)
				}
				Cu[k,iunif] <- Cu[iunif,k]
			}
		}
		du <- det(Cu)
		Deter[compt] <- dinit
		if((du - dinit) >= 0){
			Deter[compt] <- du
			p <- pu
			C <- Cu
			Distance <- Distu
			dinit <- du
		}
	
	 }# End while

	# Outputs:
 	return(list(n=n,dimension=dimension,range=range,niter_max=niter_max,design_init=p_init,design=p,det_init=Deter[1],det_end=dinit,seed=seed))
}

mindist <- function(design){
	# Compute the Mindist criterion
	# A higher MINDIST corresponds to a more regular
	# scaterring of design points
	# J.FRANCO, 2006.10.23
	# input : design of n experiments

	X <- as.matrix(design)
	n <- dim(X)[1]
	# To check the experimental region
	if ( min(X)<0 || max(X)>1 ){
		warning("The design is rescaling into the unit cube [0,1]^d.")
		M <- apply(X,2,max)
		m <- apply(X,2,min)
		for (j in 1:dim(X)[2]){
			X[,j] <- (X[,j]-m[j])/(M[j]-m[j])
		}	
	}
	
	Distance <- as.matrix(dist(X))
	diag(Distance)<-1.0E30
	# Compute the min on the columns
	min1 <- apply(Distance,2,min)

	return(min(min1))
}

\name{wspDesign}
\alias{wspDesign}

\title{WSP algorithm}

\description{The WSP (Wooton, Sergent, Phan-Tan-Luu) algorithm is an iterative algorithm based on suppression of some experiments from an initial design in each step. WSP leads to a space filling design}

\usage{wspDesign(design, dmin, init = "center")}

\arguments{
  \item{design}{a matrix (or a data.frame) corresponding to the design of experiments}
  \item{dmin}{a minimum bound for mindist value of the final design} 
  \item{init}{defines the initialization point (input coordinates) of the algorithm: 
  "center" (default value) takes the central point of the domain
  "random" takes a random point inside the domain}
  }


\details{
WSP enables to create a design D which is such that mindist(D)>dmin. 
However, it cannot assess the number of experiments. 
Similarly to straussDesign function, WSP is a powerful algorithm to construct 
space filling designs in high dimension
} 


\value{A list containing:
  \item{InitialDesign}{the starting design}
  \item{dmin}{minimum bound for mindist value of the final design}
  \item{design}{the matrix of the final design}
  \item{ResidualDesign}{the matrix of the residual design (points of 
  InitialDesign not in design)}
  }

\references{

J. Santiago, M. Claeys-Bruno, M.Sergent (2012). Construction of space filling 
designs using WSP algorithm for high dimensional spaces,
\emph{Chenometrics and Intelligent Laboratory Systems}, 113:26-31.

}

\author{G. Damblin & B. Iooss}


\examples{
dimension <- 2
n <- 100
X <- matrix(runif(n*dimension), n, dimension)
m <- wspDesign(X, 0.1)
plot(m$design)
xDRDN(m, letter = "T", dgts = 2, range = c(-10, 10))
}

\keyword{design}

\name{mindist}
\alias{mindist}
\title{Mindist measure}
\description{Compute the \code{mindist} criterion (also called maximin)}
\usage{mindist(design)}

\arguments{
  \item{design}{a matrix (or a data.frame) representing the design of experiments in the unit cube [0,1]\eqn{^d}. If this last condition is not fulfilled, a transformation into [0,1]\eqn{^{d}} is applied before the computation of the criteria.}
}

\details{The mindist criterion is defined by
\deqn{mindist= \min_{x_{i}\in X} \left( \gamma_{i} \right)}{mindist = min (g_1, ... g_n)}
where \eqn{\gamma_{i}}{g_i} is the minimal distance between the point \eqn{x_{i}}{x_i}
and the other points \eqn{x_{k}}{x_k} of the \code{design}.

A higher value corresponds to a more regular scaterring of design points.
}

\value{A real number equal to the value of the mindist criterion for the \code{design}.}

\references{

Gunzburer M., Burkdart J. (2004), Uniformity measures for point samples in hypercubes, \url{https://people.sc.fsu.edu/~jburkardt/}.

Jonshon M.E., Moore L.M. and Ylvisaker D. (1990), Minmax and maximin distance designs, \emph{J. of Statis. Planning and Inference}, 26, 131-148.

Chen V.C.P., Tsui K.L., Barton R.R. and Allen J.K. (2003), A review of design and modeling in computer experiments, \emph{Handbook of Statistics}, 22, 231-261.
}

\author{J. Franco}

\seealso{ 
other distance criteria like \code{\link{meshRatio}} and \code{\link{phiP}}, 
discrepancy measures provided by \code{\link{discrepancyCriteria}}.}

\examples{
dimension <- 2
n <- 40
X <- matrix(runif(n*dimension), n, dimension)
mindist(X)
}

\keyword{design}

\name{DiceDesign-package}
\alias{DiceDesign-package}
\alias{DiceDesign}
\docType{package}
\title{Designs of Computer Experiments}
\description{Space-Filling Designs (SFD) and space-filling criteria (distance-based and uniformity-based).}
\details{
%\tabular{ll}{
%	Package: \tab DiceDesign\cr
%	Type: \tab Package\cr
%	Version: \tab 1.7\cr
%	Date: \tab 2015-06-15\cr
%	License: \tab GPL-3\cr
%}
This package provides tools to create some specific Space-Filling Design (SFD) and to test their quality:
\itemize{
    \item Latin Hypercube designs (randomized or centered)
    \item Strauss SFD and Maximum entropy SFD, WSP designs
    \item Optimal (low-discrepancy and maximin) Latin Hypercube desigsn by simulated annealing and genetic algorithms,
    \item Orthogonal and Nearly Orthogonal Latin Hypercube designs,
    \item Discrepancies criteria, distance measures,
    \item Minimal spanning tree criteria,
    \item Radial scanning statistic
}
}

\author{
J. Franco, D. Dupuy, O. Roustant, P. Kiener, G. Damblin and B. Iooss. Thanks to A. Jourdan for discussions about OA131.

Maintainer: Celine Helbert \email{Celine.Helbert@ec-lyon.fr} 
}

\note{
Part of this work was conducted on 2006-2009 within the frame of the DICE (Deep Inside Computer Experiments) Consortium between ARMINES, Renault, EDF, IRSN, ONERA and TOTAL S.A. (http://dice.emse.fr/).

In this package, only Faure's sequence is implemented. Note that the \pkg{randtoolbox} package provides the following quasi random sequences: the Sobol sequence, the Halton (hence Van Der Corput) sequence and the Torus sequence (also known as Kronecker sequence). Note also that the \pkg{lhs} package provides other types of algorithms to compute optimized LHS.
}

\references{
Cioppa T.M., Lucas T.W. (2007). Efficient nearly orthogonal and space-filling Latin hypercubes. \emph{Technometrics} 49, 45-55.

Damblin G., Couplet M., and Iooss B. (2013). Numerical studies of space filling designs: optimization of Latin Hypercube Samples and subprojection properties, \emph{Journal of Simulation}, 7:276-289, 2013.

De Rainville F.-M., Gagne C., Teytaud O., Laurendeau D. (2012). Evolutionary optimization of low-discrepancy sequences. \emph{ACM Transactions on Modeling and Computer Simulation (TOMACS)}, 22(2), 9.

Dupuy D., Helbert C., Franco J. (2015), DiceDesign and DiceEval: Two R-Packages for Design and Analysis of Computer Experiments, \emph{Journal of Statistical Software}, \bold{65}(11), 1--38.

Fang K.-T., Li R. and Sudjianto A. (2006) Design and Modeling for Computer Experiments, \emph{Chapman & Hall}.

Fang K-T., Liu M-Q., Qin H. and Zhou Y-D. (2018) Theory and application of uniform experimental designs. \emph{Springer}.

Nguyen N.K. (2008) A new class of orthogonal Latinhypercubes, \emph{Statistics and Applications}, Volume 6, issues 1 and 2, pp.119-123.

Owen A.B. (2020), On dropping the first Sobol point, \url{https://arxiv.org/abs/2008.08051}.

Roustant O., Franco J., Carraro L., Jourdan A. (2010), A radial scanning statistic for selecting space-filling designs in computer experiments, \emph{MODA-9 proceedings}.

Santner T.J., Williams B.J. and Notz W.I. (2003) The Design and Analysis of Computer Experiments, \emph{Springer}, 121-161.
}

\examples{
# **********************
# Designs of experiments
# **********************

# A maximum entropy design with 20 points in [0,1]^2
p <- dmaxDesign(20,2,0.9,200)
plot(p$design,xlim=c(0,1),ylim=c(0,1))

# Change the dimnames, adjust to range (-10, 10) and round to 2 digits
xDRDN(p, letter = "T", dgts = 2, range = c(-10, 10))

# ************************
# Criteria: L2-discrepancy
# ************************
dp <- discrepancyCriteria(p$design,type=c('L2','C2'))
# Coverage measure
covp <- coverage(p$design)

# *******************************
# Criteria: Minimal Spanning Tree
# *******************************
mstCriteria(p$design,plot2d=TRUE)

# ****************************************************************
# Radial scanning statistic: Detection of defects of Sobol designs
# ****************************************************************

# requires randtoolbox package
library(randtoolbox)

# in 2D
rss <- rss2d(design=sobol(n=20, dim=2), lower=c(0,0), upper=c(1,1),
	type="l", col="red")

# in 8D. All pairs of dimensions are tried to detect the worst defect
# (according to the specified goodness-of-fit statistic).
d <- 8
n <- 10*d
rss <- rss2d(design=sobol(n=n, dim=d), lower=rep(0,d), upper=rep(1,d),
	type="l", col="red")

# avoid this defect with scrambling ?
#    1. Faure-Tezuka scrambling (type "?sobol" for more details and options)
rss <- rss2d(design=sobol(n=n, dim=d, scrambling=2), lower=rep(0,d),
	upper=rep(1,d), type="l", col="red")
#    2. Owen scrambling
rss <- rss2d(design=sobol(n=n, dim=d, scrambling=1), lower=rep(0,d),
	upper=rep(1,d), type="l", col="red")

}
\keyword{package}

\name{OA131_scrambled}
\alias{OA131_scrambled}
\docType{data}
\title{ A scrambled 3D orthogonal array of strength 2}
\description{
This design is obtained by adding a uniform noise to each coordinate of the orthogonal array OA131.
}

\usage{data(OA131_scrambled)}
\format{
  A data frame with 49 observations on the following 3 variables.
  \describe{
    \item{\code{x1}}{first coordinate}
    \item{\code{x2}}{second coordinate}
    \item{\code{x3}}{third coordinate}
  }
}

\source{
Roustant O., Franco J., Carraro L., Jourdan A. (2010), A radial scanning statistic for selecting space-filling designs in computer experiments, MODA-9 proceedings.
} 

\examples{
data(OA131)
data(OA131_scrambled)

pairs(OA131, xlim=c(0,1), ylim=c(0,1))
pairs(OA131_scrambled, xlim=c(0,1), ylim=c(0,1))
}

\keyword{datasets}

\name{OA131}
\alias{OA131}
\docType{data}
\title{ A 3D orthogonal array of strength 2}
\description{
  A 3-dimensional linear orthogonal array (OA) of strength 2 with 49 points. The design points are equally spaced into 2 dimensional coordinate planes. However by construction, such OAs satisfy a linear relation, here: \code{x1 + 3*x2 + x3 = 0 (mod 7)}. As a consequence, the design points are contained in parallel planes orthogonal to (1,3,1). Actually, they are also contained in parallel planes orthogonal to other directions, as (2,-1,2) or (3,2,3), since the congruence relation leads to \code{2*x1 - x2 + 2*x3 = 0 (mod 7)} or \code{3*x1 + 2*x2 + 3*x3 = 0 (mod 7)}. For instance, they are contained in 4 parallel planes orthogonal to (2,-1,2).
}

\usage{data(OA131)}
\format{
  A data frame with 49 observations on the following 3 variables.
  \describe{
    \item{\code{x1}}{first coordinate}
    \item{\code{x2}}{second coordinate}
    \item{\code{x3}}{third coordinate}
  }
}

\source{ 
Roustant O., Franco J., Carraro L., Jourdan A. (2010), A radial scanning statistic for selecting space-filling designs in computer experiments, MODA-9 proceedings.
}

\examples{
data(OA131)

# centering and reducing to [0,1]^3
OA <- (OA131 + 0.5)/7
pairs(OA, xlim=c(0,1), ylim=c(0,1))

\dontrun{
library(lattice)
cloud(x3~x1+x2, data=OA, xlim=c(0,1), ylim=c(0,1), zlim=c(0,1), 
      screen = list(z = 50, x = -70, y = 0))}
}

\keyword{datasets}

\name{coverage}
\alias{coverage}
\title{Coverage}
\description{Compute the coverage measure}
\usage{coverage(design)}

\arguments{
  \item{design}{a matrix (or a data.frame) representing the design of experiments representing the design of experiments in the unit cube [0,1]\eqn{^d}. If this last condition is not fulfilled, a transformation into [0,1]\eqn{^{d}} is applied before the computation of the criteria.}
}

\details{The coverage criterion is defined by
\deqn{coverage=\frac{1}{\bar{\gamma}} \left[ \frac{1}{n} \sum_{i=1}^{n}
	\left( \gamma_{i} - \bar{\gamma} \right)^2 \right]^{1/2}}{coverage)
	=1/gMean *[ 1/n * [( g_1 - gMean )^2 + ... + (g_n - gMean)^2] ]^(1/2)}
where \eqn{\gamma_{i}}{g_i} is the minimal distance between the point \eqn{x_{i}}{x_i}
and the other points of the \code{design} and \eqn{\bar{\gamma}}{gMean} is
the mean of the \eqn{\gamma_{i}}{g_i}. 

Note that for a regular mesh, \code{cov}=0. Then, a small value of \code{cov} means that the design is close to a regular grid.}
\value{A real number equal to the value of the coverage criterion for the \code{design}.}

\references{Gunzburer M., Burkdart J. (2004) \emph{Uniformity measures for point samples in hypercubes}, \url{https://people.sc.fsu.edu/~jburkardt/}.}

\author{J. Franco}

\seealso{other distance criteria like \code{\link{meshRatio}}, \code{\link{phiP}} and \code{\link{mindist}}.

discrepancy measures provided by \code{\link{discrepancyCriteria}}.}

\examples{
dimension <- 2
n <- 40
X <- matrix(runif(n*dimension), n, dimension)
coverage(X)
}

\keyword{design}

\name{nolhDesign}
\alias{nolhDesign}

\title{Cioppa's Nearly Orthogonal Latin Hypercube Designs}
\description{
     This function generates a NOLH design of dimension 2 to 29 and normalizes it to the selected range. The design is extracted from Cioppa's \code{\link{NOLHdesigns}} list.
      
}
\usage{
nolhDesign(dimension, range = c(0, 1))
}

\arguments{
  \item{dimension }{ number of input variables }
  \item{range     }{ the scale (min and max) of the inputs. Range (0, 0) and (1, 1) are special cases and call integer ranges \eqn{(-m, m)} and \eqn{(0, 2m)}. See the examples}
}

\value{
A list with components:
  \item{n              	}{the number of lines/experiments}
  \item{dimension      	}{the number of columns/input variables}
  \item{design  		}{the design of experiments}
}

\author{T.M. Cioppa for the designs. P. Kiener for the R code.}

\seealso{
Cioppa's list \code{\link{NOLHdesigns}}. Other NOLH and OLH designs: \code{\link{nolhdrDesign}}, \code{\link{olhDesign}}.
}

\examples{
## Classical normalizations
nolhDesign(8, range = c(1, 1))
nolhDesign(8, range = c(0, 0))
nolhDesign(8, range = c(0, 1))
nolhDesign(8, range = c(-1, 1))

## Change the dimnames, adjust to range (-10, 10) and round to 2 digits
xDRDN(nolhDesign(8), letter = "T", dgts = 2, range = c(-10, 10))

## A list of designs
lapply(5:9, function(n) nolhDesign(n, range = c(-1, 1))$design)
}

\keyword{design}

\name{maximinSA_LHS}
\alias{maximinSA_LHS}

\title{Simulated annealing (SA) routine for Latin Hypercube Sample (LHS) optimization via phiP criteria}
\description{The objective is to produce maximin LHS. SA is an efficient algorithm to produce space-filling designs.}
\usage{
maximinSA_LHS(design, T0=10, c=0.95, it=2000, p=50, profile="GEOM", Imax=100)
}

\arguments{
  \item{design}{a matrix (or a data.frame) corresponding to the design of experiments}
  \item{T0}{The initial temperature of the SA algorithm}
  \item{c}{A constant parameter regulating how the temperature goes down}
  \item{it}{The number of iterations}
  \item{p}{power required in phiP criterion}
  \item{profile}{The temperature down-profile, purely geometric called "GEOM", geometrical according to the Morris algorithm called "GEOM_MORRIS" or purely linear called "LINEAR"}
  \item{Imax}{A parameter given only if you choose the Morris down-profile. It adjusts the number of iterations without improvement before a new elementary perturbation}
  }


\details{This function implements a classical routine to produce optimized LHS. It is based on the work of Morris and Mitchell (1995). They have proposed a SA version for LHS optimization according to mindist criterion. Here, it has been adapted to the \code{phiP} criterion. It has been shown (Pronzato and Muller, 2012, Damblin et al., 2013) that optimizing phiP is more efficient to produce maximin designs than optimizing mindist. When \eqn{p} tends to infinity, optimizing a design with \code{phi_p} is equivalent to optimizing a design with \code{mindist}.}

\value{A list containing:
  \item{InitialDesign}{the starting design}
  \item{T0}{the initial temperature of the SA algorithm}
  \item{c}{the constant parameter regulating how the temperature goes down}
  \item{it}{the number of iterations}
  \item{p}{power required in phiP criterion}
  \item{profile}{the temperature down-profile}
  \item{Imax}{The parameter given in the Morris down-profile}
  \item{design}{the matrix of the final design (maximin LHS)}
  \item{critValues}{vector of criterion values along the iterations}
  \item{tempValues}{vector of temperature values along the iterations}
  \item{probaValues}{vector of acceptation probability values along the iterations}
}

\references{

Damblin G., Couplet M., and Iooss B. (2013). Numerical studies of space filling designs: optimization of Latin Hypercube Samples and subprojection properties, \emph{Journal of Simulation,} 7:276-289, 2013. 

M. Morris and J. Mitchell (1995) Exploratory designs for computationnal experiments. Journal of 
Statistical Planning and Inference, 43:381-402.

R. Jin, W. Chen and A. Sudjianto (2005) An efficient algorithm for constructing optimal design
of computer experiments. Journal of Statistical Planning and Inference, 134:268-287.

Pronzato, L. and Muller, W. (2012). Design of computer experiments: space filling and beyond, \emph{Statistics and Computing,} 22:681-701.
}

\author{G. Damblin & B. Iooss}

\seealso{
Latin Hypercube Sample (\code{\link{lhsDesign}}), 
discrepancy criteria (\code{\link{discrepancyCriteria}}), 
geometric criterion (\code{\link{mindist}}, \code{\link{phiP}}), 
optimization (\code{\link{discrepSA_LHS}}, \code{\link{maximinESE_LHS}}, \code{\link{discrepESE_LHS}})
}


\examples{
dimension <- 2
n <- 10
X <- lhsDesign(n ,dimension)$design
Xopt <- maximinSA_LHS(X, T0=10, c=0.99, it=2000)
plot(Xopt$design)
plot(Xopt$critValues, type="l")
plot(Xopt$tempValues, type="l")

\dontrun{
  Xopt <- maximinSA_LHS(X, T0=10, c=0.99, it=1000, profile="GEOM_MORRIS")
}
}

\keyword{design}

\name{maximinESE_LHS}
\alias{maximinESE_LHS}

\title{Enhanced Stochastic Evolutionnary (ESE) algorithm for Latin Hypercube Sample (LHS) optimization via phiP criteria}
\description{The objective is to produce maximin LHS. ESE is a powerful genetic algorithm allowing to produce space-filling designs.}
\usage{
maximinESE_LHS(design, T0=0.005*phiP(design,p=50), inner_it=100, J=50, it=1, p=50)
}

\arguments{
  \item{design}{a matrix (or a data.frame) corresponding to the design of experiments.}
  \item{T0}{The initial temperature of the ESE algorithm}
  \item{inner_it}{The number of iterations for inner loop}
  \item{J}{The number of new proposed LHS inside the inner loop}
  \item{it}{The number of iterations for outer loop}
  \item{p}{power required in phiP criterion}
  }


\details{This function implements a stochastic algorithm (ESE) to produce optimized LHS. It is based on Jin et al works (2005).}


\value{A list containing:
  \item{InitialDesign}{the starting design}
  \item{T0}{the initial temperature of the ESE algorithm}
  \item{inner_it}{the number of iterations for inner loop}
  \item{J}{the number of new proposed LHS inside the inner loop}
  \item{it}{the number of iterations for outer loop}
  \item{p}{power required in phiP criterion}
  \item{design}{the matrix of the final design (maximin LHS)}
  \item{critValues}{vector of criterion values along the iterations}
  \item{tempValues}{vector of temperature values along the iterations}
  \item{probaValues}{vector of acceptation probability values along the iterations}
}

\references{

Damblin G., Couplet M., and Iooss B. (2013). Numerical studies of space filling designs: optimization of Latin Hypercube Samples and subprojection properties, \emph{Journal of Simulation,} 7:276-289, 2013.

M. Morris and J. Mitchell (1995) Exploratory designs for computationnal experiments. Journal of 
Statistical Planning and Inference, 43:381-402.

R. Jin, W. Chen and A. Sudjianto (2005) An efficient algorithm for constructing optimal design
of computer experiments. Journal of Statistical Planning and Inference, 134:268-287.

Pronzato, L. and Muller, W. (2012). Design of computer experiments: space filling and beyond, \emph{Statistics and Computing,} 22:681-701.
}

\author{G. Damblin & B. Iooss}

\seealso{
Latin Hypercube Sample (\code{\link{lhsDesign}}), 
discrepancy criteria (\code{\link{discrepancyCriteria}}), 
geometric criterion (\code{\link{mindist}}, \code{\link{phiP}}), 
optimization (\code{\link{maximinSA_LHS}}, \code{\link{discrepESE_LHS}}, \code{\link{discrepSA_LHS}})
}


\examples{
dimension <- 2
n <- 10
X <- lhsDesign(n, dimension)$design
Xopt <- maximinESE_LHS(X, T0=0.005*phiP(X), inner_it=100, J=50, it=2)
plot(Xopt$design)
plot(Xopt$critValues, type="l")

}

\keyword{design}

\name{meshRatio}
\alias{meshRatio}
\title{MeshRatio measure}
\description{ The \code{meshRatio} criterion is the ratio between the
maximimum and the minimum distance between two points of the experimental design.
}
\usage{meshRatio(design)}

\arguments{
  \item{design}{a matrix (or a data.frame) representing the design of experiments in the unit cube [0,1]\eqn{^d}. If this last condition is not fulfilled, a transformation into [0,1]\eqn{^{d}} is applied before the computation of the criteria.}
}

\details{The \code{meshRatio} criterion is defined by
\deqn{meshRatio=\frac{\max_{1\leq i \leq n} \gamma_{i}}{\min_{1\leq i \leq n}
\gamma_{i}}}{meshRatio(\code{design})=max(g_1, ..., g_n)/min(g_1, ..., g_n)}
where \eqn{\gamma_{i}}{g_i} denotes the minimal distance between the point
\eqn{x_{i}}{x_i} and the other points of the \code{design}.

Note that for a regular mesh, \code{meshRatio}=1.
}

\value{
A real number equal to the value of the meshRatio criterion for the \code{design}.
}

\references{
Gunzburer M. and Burkdart J. (2004), Uniformity measures for point samples in hypercubes, \url{https://people.sc.fsu.edu/~jburkardt/}.
}

\author{J. Franco}

\seealso{Other distance criteria like \code{\link{meshRatio}}, \code{\link{phiP}} and \code{\link{mindist}}.

Discrepancy measures provided by \code{\link{discrepancyCriteria}}.}

\examples{
dimension <- 2
n <- 40
X <- matrix(runif(n*dimension), n, dimension)
meshRatio(X)
}

\keyword{design}

\name{discrepSA_LHS}
\alias{discrepSA_LHS}

\title{Simulated annealing (SA) routine for Latin Hypercube Sample (LHS) optimization via L2-discrepancy criteria}
\description{The objective is to produce low-discrepancy LHS. SA is an efficient algorithm to produce space-filling designs. It has been adapted here to main discrepancy criteria.}

\usage{
discrepSA_LHS(design, T0=10, c=0.95, it=2000, criterion="C2", profile="GEOM", Imax=100)
}

\arguments{
  \item{design}{a matrix (or a data.frame) corresponding to the design of experiments}
  \item{T0}{The initial temperature}
  \item{c}{A constant parameter regulating how the temperature goes down}
  \item{it}{The number of iterations}
  \item{criterion}{The criterion to be optimized. One can choose three different L2-discrepancies: the C2 (centered) discrepancy ("C2"), the L2-star discrepancy ("L2star") and the W2 (wrap-around) discrepancy ("W2")}
  \item{profile}{The temperature down-profile, purely geometric called "GEOM", geometrical according to the Morris algorithm called "GEOM_MORRIS" or purely linear called "LINEAR"}
  \item{Imax}{A parameter given only if you choose the Morris down-profile. It adjusts the number of iterations without improvement before a new elementary perturbation}
  }


\details{This function implements a classical routine to produce optimized LHS. It is based on the work of Morris and Mitchell (1995). They have proposed a SA version for LHS optimization according to mindist criterion. Here, it has been adapted to some discrepancy criteria taking in account new ideas about the reevaluations of a discrepancy value after a LHS elementary perturbation (in order to avoid computing all terms in the discrepancy formulas).}


\value{A list containing:
  \item{InitialDesign}{the starting design}
  \item{T0}{the initial temperature of the SA algorithm}
  \item{c}{the constant parameter regulating how the temperature goes down}
  \item{it}{the number of iterations}
   \item{criterion}{the criterion to be optimized}
  \item{profile}{the temperature down-profile}
  \item{Imax}{The parameter given in the Morris down-profile}
  \item{design}{the matrix of the final design (low-discrepancy LHS)}
  \item{critValues}{vector of criterion values along the iterations}
  \item{tempValues}{vector of temperature values along the iterations}
  \item{probaValues}{vector of acceptation probability values along the iterations}
}

\references{

Damblin G., Couplet M., and Iooss B. (2013). Numerical studies of space filling designs: optimization of Latin Hypercube Samples and subprojection properties, \emph{Journal of Simulation,} 7:276-289, 2013.

M. Morris and J. Mitchell (1995) Exploratory designs for computationnal experiments. Journal of 
Statistical Planning and Inference, 43:381-402.

R. Jin, W. Chen and A. Sudjianto (2005) An efficient algorithm for constructing optimal design
of computer experiments. Journal of Statistical Planning and Inference, 134:268-287.

}

\author{G. Damblin & B. Iooss}

\seealso{Latin Hypercube Sample(\code{\link{lhsDesign}}),discrepancy criteria(\code{\link{discrepancyCriteria}}), geometric criterion (\code{\link{mindist}}\code{\link{phiP}}), optimization (\code{\link{maximinSA_LHS}},\code{\link{maximinESE_LHS}} ,\code{\link{discrepESE_LHS}})}


\examples{
dimension <- 2
n <- 10
X <- lhsDesign(n, dimension)$design

## Optimize the LHS with C2 criterion
Xopt <- discrepSA_LHS(X, T0=10, c=0.99, it=2000, criterion="C2")
plot(Xopt$design)
plot(Xopt$critValues, type="l")

## Optimize the LHS with C2 criterion and GEOM_MORRIS profile
\dontrun{
Xopt2 <- discrepSA_LHS(X, T0=10, c=0.99, it=1000, criterion="C2", profile="GEOM_MORRIS")
plot(Xopt2$design)
}
}

\keyword{design}

\name{discrepancyCriteria}
\alias{discrepancyCriteria}

\title{Discrepancy measure}
\description{Compute discrepancy criteria. }
\usage{discrepancyCriteria(design,type='all')}

\arguments{
  \item{design}{a matrix (or a data.frame) corresponding to the design of experiments.
  The discrepancy criteria are computed for a design in the unit cube [0,1]\eqn{^d}.
  If this condition is not satisfied the design is automatically rescaled.}
  \item{type}{type of discrepancies (single value or vector) to be computed:
	\tabular{ll}{
		\code{'all'} \tab all type of discrepancies (default) \cr
  	\code{'C2'} \tab centered L2-discrepancy \cr
  	\code{'L2'} \tab L2-discrepancy \cr
  	\code{'L2star'} \tab L2star-discrepancy \cr
		\code{'M2'} \tab modified L2-discrepancy \cr
		\code{'S2'} \tab symmetric L2-discrepancy \cr
		\code{'W2'} \tab wrap-around L2-discrepancy \cr
		\code{'Mix2'} \tab mixture L2-discrepancy \cr
	}
  }
}

\details{The discrepancy measures how far a given distribution of points deviates
from a perfectly uniform one. Different L2 discrepancies are available in DiceDesign.
For example, if we denote by \eqn{Vol(J)} the volume of a subset \eqn{J} of \eqn{[0; 1]^d} and \eqn{A(X; J)} the number of points of \eqn{X} falling in \eqn{J}, the \eqn{L2} discrepancy is:
\deqn{D_{L2} (X) = \left[ \int_{[0,1]^{2d}}{} \left( \frac{A(X,J_{a,b})}{n} - Vol (J_{a,b}) \right)^{2} da db \right]^{1/2}}{DL2 (X)^2 = \int_{[0,1]^d} [(A(X,J_{a,b})/n - Vol(J_{a,b})]^2 da db}
where \eqn{a = (a_{1}; ... ; a_{d})'}{a = (a1; ... ; ad)'}, \eqn{b = (b_{1};...; b_{d})'}{b = (b1;...; bd)'} and \eqn{J_{a,b} =
[a_{1}; b_{1}) \times ... \times [a_{d};b_{d})}{J_{a,b} =
[a1; b1) X ... X [ad;bd)}. The other L2-discrepancies are defined according to the same principle with different form from the subset \eqn{J}.
Among all the possibilities, discrepancyCriteria implements only the L2 discrepancies because it can be expressed analytically even for high dimension.

Centered L2-discrepancy is computed using the analytical expression done by Hickernell (1998). The user will refer to Pleming and Manteufel (2005) to have more details about the wrap around discrepancy.
}

\value{A list containing the L2-discrepancies of the \code{design}.}

\references{
Fang K.T, Li R. and Sudjianto A. (2006) Design and Modeling for
	Computer Experiments, \emph{Chapman & Hall}.

Fang K-T., Liu M-Q., Qin H. and Zhou Y-D. (2018) Theory and application of uniform
experimental designs. \emph{Springer}.

Franco J. (2008) Planification d'experiences numerique en phase exploratoire pour
la simulation des phenomenes complexes, \emph{PhD thesis, Ecole Nationale Superieure
des Mines de Saint Etienne}.

Hickernell F.J. (1998) A generalized discrepancy and quadrature error bound.
	\emph{Mathematics of Computation}, \bold{67}, 299-322.

Pleming J.B. and Manteufel R.D. (2005) \emph{Replicated Latin Hypercube Sampling},
46th Structures, Structural Dynamics & Materials Conference, 16-21 April 2005, Austin
(Texas) -- AIAA 2005-1819.
}

\author{J. Franco, D. Dupuy & B. Iooss}

\seealso{distance criteria (\code{\link{coverage}}, \code{\link{meshRatio}},
\code{\link{mindist}} and \code{\link{phiP}})}

\examples{
dimension <- 2
n <- 40
X <- matrix(runif(n*dimension), n, dimension)
discrepancyCriteria(X)
}

\keyword{design}

\name{dmaxDesign}
\alias{dmaxDesign}
\title{ Maximum Entropy Designs }
\description{
  Space-Filling Designs with \eqn{n}{n} experiments based on covariance matrix in
  [0,1]\eqn{^d}{^d}.
}
\usage{
dmaxDesign(n, dimension, range, niter_max=1000, seed=NULL)
}

\arguments{
  \item{n        }{ number of experiments }
  \item{dimension}{ number of variables }
  \item{range    }{ range of variogram }
  \item{niter_max}{ number of iterations}
    \item{seed	 }{ seed used to generate uniform design}
}

\details{Maximum entropy design is a kind of optimal design based
on Shannon's definition of entropy as the amount of information.
Originally, maximum entropy sampling was proposed by Shewry and
Wynn (1987). The goal of the design is to maximize the entropy
defined as the determinant of the correlation matrix using a Fedorov-Mitchell
exchange algorithm.

The spatial correlation matrix is defined by \eqn{C=\left( \rho_{ij}
\right)}{C=(r[i,j])}:
\tabular{ll}{
	\eqn{\rho_{ij}=1-\gamma\left(h_{ij}\right)}{r[i,j]=1-gamma(h[i,j])} \tab if \eqn{h_{ij}\leq a}{h[i,j] <=a}, \cr
	\eqn{\rho_{ij}=0}{r[i,j]=0} \tab if \eqn{h_{ij}>a}{if h_{ij}>a},}
where \eqn{h_{ij}}{h[i,j]} is the distance between \eqn{x_{i}}{xi} and
\eqn{x_{j}}{xj}, \eqn{a}{a} denotes the range of the variogram and
\eqn{\gamma}{gamma} is a spherical variogram:
\deqn{\gamma(h) = 1.5 \frac{h}{a}- 0.5\left(\frac{h}{a}\right)^3
\textnormal{ for } h \leq a}{gamma(h)= 1.5*(h/a)- 0.5*(h/a)^3 for h>=a.}
}

\value{A list with components:
  \item{n              	}{the number of points}
  \item{design			}{the design of experiments}
  \item{dimension      	}{the number of variables}
  \item{range          	}{the range of the variogram}
  \item{niter_mx		}{the number of iterations}
  \item{design_init		}{the initial distribution}
  \item{det_init       	}{the value of the determinant for the initial distribution}
  \item{det_end        	}{the value of the determinant at the end of the procedure}
  \item{seed			}{the value of the seed}
}

\references{Currin C., Mitchell T., Morris M. and Ylvisaker D. (1991)
\emph{Bayesian Prediction of Deterministic Functions With Applications
to the Design and Analysis of Computer Experiments}, American
Statistical Association, \bold{86}, 416, 953-963.

Shewry, M. C. and Wynn and H. P. (1987) \emph{Maximum entropy sampling},
Journal of Applied Statistics 14, 165-170.}

\author{J. Franco}

\examples{
n <- 20
dimension <- 2
range <-0.9
niter_max <- 200
out <- dmaxDesign(n, dimension, range, niter_max)

## Change the dimnames, adjust to range (-10, 10) and round to 2 digits
xDRDN(out, letter = "T", dgts = 2, range = c(-10, 10))
}

\keyword{design}

\name{phiP}
\alias{phiP}

\title{phiP criterion}
\description{Compute the \eqn{\phi_p} criterion (strongly linked to mindist criterion)}
\usage{phiP(design, p=50)}

\arguments{
  \item{design}{a matrix (or a data.frame) corresponding to the design of experiments.}
  \item{p}{the "p" in the Lp norm which is taken} 
  }


\details{The \eqn{\phi_p} criterion is defined by the \eqn{L_p} norm of the sum of the inverses of the design inter-point euclidean distances:
\deqn{\phi_{p}=\left[\sum_{i,j=1\ldots N,i<j}\,\,d_{ij}^{-p}\right]^{\frac{1}{p}}}{}

A higher value corresponds to a more regular scaterring of design points.

When \eqn{p} tends to infinity, optimizing a design with \eqn{\phi_p} is equivalent to optimizing a design with \code{mindist}.
}


\value{A real number equal to the value of the \eqn{\phi_p} criterion for the \code{design}.}

\references{

Damblin G., Couplet M., and Iooss B. (2013). Numerical studies of sapce filling designs: optimization of Latin Hypercube Samples and subprojection properties, \emph{Journal of Simulation}, 7:276-289, 2013.

Fang K.-T., Li R. and Sudjianto A. (2006). Design and Modeling for Computer Experiments, \emph{Chapman & Hall}.

Pronzato, L. and Muller, W. (2012). Design of computer experiments: space filling and beyond, \emph{Statistics and Computing}, 22:681-701.

}

\author{G. Damblin & B.Iooss}

\seealso{geometric criterion (\code{\link{mindist}})} 


\examples{
dimension <- 2
n <- 40
X <- matrix(runif(n*dimension), n, dimension)
phiP(X)
}

\keyword{design}

\name{factDesign}
\alias{factDesign}
\title{Full Factorial Designs}
\description{
Create a factorial design with \eqn{n}{n} = pow(levels,dimension) experiments
in [0,1]\eqn{^d}{^d}.
}

\usage{
factDesign(dimension, levels)
}

\arguments{
  \item{dimension}{an integer given the number of input variables }
  \item{levels   }{an integer given the number of levels}
}

\details{It is possible to take a different number of levels for
any factor. In this case, the argument \code{levels} should be a
vector.
}

\value{\code{factDesign} returns a list containing all the
input arguments detailed before, plus the following components:
  \item{n         }{the number of experiments}
  \item{design    }{the design of experiments}
}
\author{G. Pujol and J. Franco}


\examples{
## First example
g1 <- factDesign(2, 7)
plot(g1$design, xlim=c(0,1), ylim=c(0,1))

## Second example
g2 <- factDesign(2, c(2,7))
plot(g2$design, xlim=c(0,1), ylim=c(0,1))

## Change the dimnames, adjust to range (-10, 10) and round to 2 digits
xDRDN(g1, letter = "T", dgts = 2, range = c(-10, 10))
xDRDN(g2, letter = "T", dgts = 2, range = c(-10, 10))
}

\keyword{design}

\name{unif.test.quantile}
\alias{unif.test.quantile}
\title{Quantile of some uniformity tests}
\description{Computes the quantile of a uniformity test at a given significance level (see available tests and levels below).}
	
\usage{
unif.test.quantile(type, n, alpha)
}

\arguments{
	\item{type}{a character indicating which test is used. The choices are the following: "greenwood", "qm" (for Quesenberry-Miller), "ks" (Kolmogorov-Smirnov), "cvm" (Cramer-Von Mises) and "V" (D+ + D- from Kolmogorov-Smirnov).}
	\item{n}{an integer equal to the sample size.}
	\item{alpha}{a real number equal to significance level. At present stage, only four values are available: 0.1, 0.05, 0.025 and 0.01.}
}

\value{A real number equal to the quantile of the specified test at significance level \code{alpha} for \code{n} observations.}

\references{ 
	D Agostino R.B., Stephens M.A. (1986), Goodness-of-fit techniques, CRC Press, New York.
}

\details{
	Modified statistics are used. For \code{alpha = 0.05}, the quantile is (see D Agostino and Stephens, 1986, section 4.4.): \code{1.358/(sqrt(n) + 0.12 + 0.11/sqrt(n))} for Kolmogorov-Smirnov and \code{0.461/(1+1/n) + 0.4/n - 0.6/n^2} for Cramer-von Mises. When the design size is \code{< 20}, the corrected value seems to be a good approximation, but the non asymptotical value should be preferred.
}

\author{O. Roustant}

\seealso{\code{\link{unif.test.statistic}}, \code{\link{rss2d}}, \code{\link{rss3d}}}

\keyword{design}

\name{xDRDN}
\alias{xDRDN}

\title{Extract a Design and Give it a Range and Dimnames}
\description{
     Extract a design contained in a list (i.e. with a \eqn{design} item), adjust the range, give it dimnames and finally round the values to a certain number of digits. Colnames will look like (\code{A,B,C}), (\code{X1,X2,X3}), (\code{X01,X02,X03}), (\code{X001,X002,X003}). 
}
\usage{
xDRDN(obj, width = 1, letter = "X", dgts = NULL, range = NULL)
}

\arguments{
  \item{obj   }{ a list that contains a \code{design} item. Matrix or data.frame are also accepted }
  \item{width }{ the digit width in colnames (to write for instance  X1, X01, X001). If 0, colnames are filled with capital and small letters (without letters \code{I} and \code{i}) up to 50 columns }
  \item{letter}{ the generic letter used in colnames }
  \item{dgts  }{ the number of digits to which the design is rounded }
  \item{range }{ a vector \code{c(min, max)} to adjust the range of the design. The default \code{NULL} keeps the original range. Special ranges \code{c(0, 0)} and \code{c(1, 1)} are not accepted }
}

\value{
A rounded matrix or a data.frame with appropriate dimnames and an adjusted range. 
}

\examples{
xDRDN(lhsDesign(5, 12))
xDRDN(lhsDesign(5, 12), width = 2, letter = "V", dgts = 2, range = c(-10, 10))
head(xDRDN(olhDesign(50, range = c(1,1)), width = 0, letter = "Z"), 3)
head(xDRDN(olhDesign(51, range = c(1,1)), width = 0, letter = "Z"), 3)
}

\name{runif.faure}
\alias{runif.faure}
\title{Low discrepancy sequence : Faure}
\description{
	Generate a Faure sequence with \eqn{n}{n} experiments in [0,1]\eqn{^d}{^d}.}
\usage{
runif.faure(n, dimension)
}

\arguments{
  \item{n        }{the number of experiments}
  \item{dimension}{the number of variables (<100)}
}

\details{A quasirandom or low discrepancy sequence, such as the
Faure, Halton, Hammersley, Niederreiter or Sobol sequences,
is "less random" than a pseudorandom number sequence, but more
useful for such tasks as approximation of integrals in higher
dimensions, and in global optimization. This is because low
discrepancy sequences tend to sample space "more uniformly"
than random numbers.

see \pkg{randtoolbox} or \pkg{fOptions} packages for other low discrepancy sequences.}

\value{\code{runif.halton} returns a list containing all the
input arguments detailed before, plus the following component:
  \item{design   }{the design of experiments}
}
\references{
Faure H. (1982), Discrepance de suites associees a un systeme de numeration (en dimension s), \emph{Acta Arith.}, 41, 337-351
}

\author{J. Franco}

\examples{
f <- runif.faure(20,2)
plot(f$design, xlim=c(0,1), ylim=c(0,1))
xDRDN(f, letter="T", dgts=2, range=c(-10, 10))
}

\keyword{design}

\name{NOLHDRdesigns}
\alias{NOLHDRdesigns}
\docType{data}

\title{List of De Rainville's Nearly Orthogonal Latin Hypercubes designs}

\description{  
  A list of the NOLH designs for 8 to 29 input variables proposed by De Rainville in 2012. These designs are said to be an improvment of Cioppa's NOLH designs as they have the same structure but better dispersion measures like the discrepancy. 
  
  This list combines the csv files published by De Rainville (see Source), centered and normalized to integer values. It is used internally by the function \code{\link{nolhdrDesign}} which provides various normalizations.

}

\usage{NOLHDRdesigns}

\format{
  A list of 22 matrices representing designs of experiments for 8 to 29 input variables: 
  \describe{
    \item{\code{nolhdr08 to nolhdr11: }}{8, 9, 10, 11 input variables, 33 experiments.}   
    \item{\code{nolhdr12 to nolhdr16: }}{12, 13, 14, 15, 16 input variables, 65 experiments.}
    \item{\code{nolhdr17 to nolhdr22: }}{17, 18, 19, 20, 21, 22 input variables, 129 experiments.}
    \item{\code{nolhdr23 to nolhdr29: }}{23, 24, 25, 26, 27, 28, 29 input variables, 257 experiments.}
  }
}

\author{F.-M. De Rainville for the designs. P. Kiener for the R code.}

\source{ 
Main website: \url{http://qrand.gel.ulaval.ca/}

The python source code: \url{https://github.com/fmder/pynolh/} 

The python package: \url{https://pypi.org/project/pynolh/}.
}

\references{
De Rainville F.-M., Gagne C., Teytaud O., Laurendeau D. (2012). Evolutionary optimization of low-discrepancy sequences. ACM Transactions on Modeling and Computer Simulation (TOMACS), 22(2), 9. 

Cioppa T.M., Lucas T.W. (2007). Efficient nearly orthogonal and space-filling Latin hypercubes. Technometrics 49, 45-55.
} 

\seealso{
The main function \code{\link{nolhdrDesign}}. Cioppa's NOLH design list: \code{\link{NOLHdesigns}}.
}

\examples{
## data(NOLHDRdesigns)

## all matrices 
names(NOLHDRdesigns)
lapply(NOLHDRdesigns, tail, 2)

## The first matrix/design
NOLHDRdesigns[["nolhdr08"]]

}

\keyword{datasets}

\name{olhDesign}
\alias{olhDesign}

\title{Nguyen's Orthogonal Latin Hypercube Designs}
\description{
   Generate the Orthogonal Latin Hypercube (OLH) designs proposed by Nguyen in 2008. These OLHs have a latin structure, an orthogonality between the main terms and the interactions (+ squares) and low correlations between the interactions (+ squares). Very larges matrices can be obtained as the number of input variables and hence the number of lines is unconstrained. When the number of input variables is a power of 2, OLHs have \eqn{d} columns and \eqn{n = 2d + 1} lines (experiments). A vertical truncature is applied when the number of input variables is not a power of 2. Various normalizations can be applied. 
      
}
\usage{
olhDesign(dimension, range = c(0, 1))
}

\arguments{
  \item{dimension }{ number of input variables }
  \item{range     }{ the scale (min and max) of the inputs. Ranges (0, 0) and (1, 1) are special cases and call integer ranges \eqn{(-d, d)} and \eqn{(0, 2d)}. See the examples}
}

\value{
A list with components:
  \item{n              	}{the number of lines/experiments}
  \item{dimension      	}{the number of columns/input variables}
  \item{design  		}{the design of experiments}
}

\author{N.K. Nguyen for the algorithm. P. Kiener for the recursive R code.}

\references{
Nguyen N.K. (2008) \emph{A new class of orthogonal Latinhypercubes}, Statistics and Applications, Volume 6, issues 1 and 2, pp.119-123.
}

\seealso{
Cioppa's and De Rainville's NOLH designs: \code{\link{nolhDesign}}, \code{\link{nolhdrDesign}}.
}

\examples{
## Classical normalizations
olhDesign(4, range = c(0, 0))
olhDesign(4, range = c(1, 1))
olhDesign(4, range = c(0, 1))
olhDesign(4, range = c(-1, 1))

## Change the dimnames, adjust to range (-10, 10) and round to 2 digits
xDRDN(olhDesign(4), letter = "T", dgts = 2, range = c(-10, 10))

## A list of designs
lapply(1:5, function(n) olhDesign(n, range = c(-1, 1))$design)
}

\keyword{design}

\name{NOLHdesigns}
\alias{NOLHdesigns}
\docType{data}

\title{List of Cioppa's Nearly Orthogonal Latin Hypercubes designs}
\description{
   A list of the NOLH designs for 2 to 29 input variables proposed by Cioppa in 2007. These designs combine a latin structure, orthogonality between the main terms and the interactions (+ squares) and reduced correlations between the interactions (+ squares). 
  
   This list combines the Excel spreadsheets published by Sanchez (see Source). It is used internally by the function \code{\link{nolhDesign}} which provides various normalizations.
}

\usage{NOLHdesigns}
\format{
  A list of 5 matrices representing designs of experiments for 8 to 29 input variables: 
  \describe{
    \item{\code{nolh2_7: }}{2 to 7 input variables, 17 experiments.}
    \item{\code{nolh8_11: }}{8 to 11 input variables, 33 experiments.}
    \item{\code{nolh12_16: }}{12 to 16 input variables, 65 experiments.}
    \item{\code{nolh17_22: }}{17 to 22 input variables, 129 experiments.}
    \item{\code{nolh23_29: }}{23 to 29 input variables, 257 experiments.}
  }
}

\author{T.M. Cioppa for the designs. P. Kiener for the R code.}

\source{
Sanchez, S. M. (2011). NOLHdesigns in Excel file. Available online at \url{https://nps.edu/web/seed/software-downloads/} 
} 

\references{
Cioppa T.M., Lucas T.W. (2007). Efficient nearly orthogonal and space-filling Latin hypercubes. Technometrics 49, 45-55.

Kleijnen, J.P.C., Sanchez S.M., T.W. Lucas and Cioppa T. M.. A user's guide to the brave new world of designing simulation experiments. INFORMS Journal on Computing 17(3): 263-289.

Ye, K. Q. (1998). Orthogonal Latin hypercubes and their application in computer experiments. J. Amer. Statist. Asso. 93, 1430-
1439.
}

\seealso{
The main function \code{\link{nolhDesign}}. De Rainville's NOLH design list: \code{\link{NOLHDRdesigns}}.
}

\examples{
## data(NOLHdesigns)

## all matrices 
names(NOLHdesigns)
lapply(NOLHDRdesigns, tail, 2)

## The first matrix/design
NOLHdesigns[["nolh2_7"]]
}

\keyword{datasets}

\name{straussDesign}
\alias{straussDesign}

\title{ Designs based on Strauss process}
\description{Space-Filling Designs based on Strauss process}
\usage{
straussDesign(n,dimension, RND, alpha=0.5, repulsion=0.001, NMC=1000,
              constraints1D=0, repulsion1D=0.0001, seed=NULL)
}

\arguments{
  \item{n        }{the number of experiments}
  \item{dimension}{the number of input variables}
  \item{RND      }{a real number which represents the radius of interaction} 
  \item{alpha    }{the potential power (default, fixed at 0.5)}
  \item{repulsion}{the repulsion parameter in the unit cube (gamma)}
  \item{NMC      }{the number of McMC iterations (this number must be large to converge)}
  \item{constraints1D	}{1 to impose 1D projection constraints, 0 otherwise}
  \item{repulsion1D }{the repulsion parameter in 1D}
  \item{seed	 }{ seed for the uniform generation of number}}

\details{Strauss designs are Space-Filling designs initially defined from Strauss process:
\deqn{\pi (X) = k \gamma^{s(X)}}{\pi (X) = k gamma^s(X)}
where \eqn{s(X)} is is the number of pairs of points \eqn{(x^{i}, x^{j})}{(xi,xj)} of the design \eqn{X = \left( x^{1}, \ldots, x^{n} \right)}{X = ( x1, ..., xn \right)} that are separated by a distance no greater than the radius of interaction \code{RND}, \eqn{k} is the normalizing constant and \eqn{\gamma}{gamma} is the repulsion parameter. This distribution corresponds to the particular case \code{alpha}=0.

For the general case, a stochastic simulation is used to construct a Markov chain which converges to a spatial density of points \eqn{\pi(X)}{\pi(X)} described by the Strauss-Gibbs potential. In practice, the Metropolis-Hastings algorithm is implemented to simulate a distribution of points which converges to the stationary law: 
\deqn{\pi (X) \propto exp(-U(X))}{\pi(X) = k exp(-U(X))}
with a potentiel \eqn{U} defined by:
\deqn{U(X) = \beta \sum_{1 \leq i < j \leq n} \varphi  \left( \| x^{i}-x^{j} \| \right)}{beta Sum_{i<j} phi(|| xi-xj ||)}
where \eqn{\beta = - \ln \gamma}{beta = -ln(gamma)}, \eqn{\varphi (h) = \left( 1 - \frac{h}{RND} \right) ^{\alpha}}{phi (h) = (1-h/RND)^{alpha}} if \eqn{h \leq}{h <=} \code{RND} and 0 otherwise.


The input parameters of \code{straussDesign} function can be interpreted as follows: 

  - \code{RND} is used to compute the number of pairs of points of the design separated by a distance no more than \code{RND}. A point is said "in interaction" with another if the spheres of radius \code{RND}/2 centered on these points intersect.

  - \code{alpha} is the potential power \eqn{\alpha}{alpha}. The case \code{alpha}=0 corresponds to Strauss process (0-1 potential). 

  - \code{repulsion} is equal to the \eqn{\gamma}{gamma} parameter of the Strauss process. Note that \eqn{\gamma}{gamma} belongs to ]0,1].

  - \code{constraints1D} allows to specify some constraints into the margin. If \code{constraints1D}==1, two repulsion parameters are needed: one for the all space (\code{repulsion}) and the other for the 1D projection (\code{repulsion1D}). Default values are \code{repulsion}=0.001 and \code{repulsion1D}=0.001. Note that the value of the radius of interaction in the one-dimensional axis is not an input parameter and is automatically fixed at \eqn{0.75/n}.}

\value{A list containing:
  \item{n         }{the number of experiments}
  \item{dimension }{the number \eqn{d} of variables}
  \item{design_init}{the initial distribution of \eqn{n} points \eqn{[0,1]^{d}}}
  \item{radius    }{the radius of interaction}
  \item{alpha	  }{the potential power alpha}
  \item{repulsion }{the repulsion parameter \eqn{\gamma}{\gamma}}
  \item{NMC       }{the number of iterations McMC}
  \item{constraints1D  }{an integer indicating if constraints on the factorial axis are imposed. If its value is different from zero, a component \code{repulsion1D} containing the value of the repulsion parameter \eqn{\gamma}{gamma} in dimension 1 is added at the list.}
  \item{design    }{the design of experiments in [0,1]\eqn{^{d}}}
  \item{seed	  }{the seed corresponding to the design}
}
\author{J. Franco}

\references{
J. Franco, X. Bay, B. Corre and D. Dupuy (2008) Planification d'experiences numeriques a partir du processus ponctuel de Strauss, \url{https://hal.science/hal-00260701/fr/}.
}

\examples{
## Strauss-Gibbs designs in dimension 2 (n=20 points)
S1 <- straussDesign(n=20, dimension=2, RND=0.2)
plot(S1$design, xlim=c(0,1), ylim=c(0,1))
theta <- seq(0,2*pi, by=2*pi/(100 - 1))
for(i in 1:S1$n){
   lines(S1$design[i,1]+S1$radius/2*cos(theta),
	     S1$design[i,2]+S1$radius/2*sin(theta), col='red')
}

## 2D-Strauss design
S2 <- straussDesign(n=20, dimension=2, RND=0.2, NMC=200,
	         constraints1D=0, alpha=0, repulsion=0.01)
plot(S2$design,xlim=c(0,1),ylim=c(0,1))

## 2D-Strauss designs with constraints on the axis
S3 <- straussDesign(n=20, dimension=2, RND=0.18, NMC=200, 
	         constraints1D=1, alpha=0.5, repulsion=0.1, repulsion1D=0.01)
plot(S3$design, xlim=c(0,1),ylim=c(0,1))
rug(S3$design[,1], side=1)
rug(S3$design[,2], side=2)

## Change the dimnames, adjust to range (-10, 10) and round to 2 digits
xDRDN(S3, letter="T", dgts=2, range=c(-10, 10))
}

\keyword{design}

\name{unif.test.statistic}
\alias{unif.test.statistic}
\title{Statistic of some uniformity tests}
\description{Computes the statistic of a uniformity test (see available tests below).}
	
\usage{
unif.test.statistic(x, type, transform=NULL)
}

\arguments{
	\item{x}{a vector containing the sample values.}
	\item{type}{a character indicating which test is used. The choices are the following: "greenwood", "qm" (for Quesenberry-Miller), "ks" (Kolmogorov-Smirnov), "cvm" (Cramer-Von Mises) and "V" (D+ + D- from Kolmogorov-Smirnov).}
		\item{transform}{ an optional character indicating what type of transformation should be applied before testing uniformity. Default is NULL.}

}

\value{A real number equal to the statistic of the specified test.}

\references{ 
	D Agostino R.B., Stephens M.A. (1986), Goodness-of-fit techniques, CRC Press, New York.
}

\author{O. Roustant}

\seealso{\code{\link{unif.test.quantile}}, \code{\link{rss2d}}}

\keyword{design}

\name{faureprimeDesign}
\alias{faureprimeDesign}
\title{A special case of the low discrepancy Faure sequence}
\description{
Generate a Faure sequence with \eqn{n=p^u-1}{n=p^u-1} experiments in [0,1]\eqn{^d}{^d} 
or other domains (see the details) where \eqn{p}{p} is the first prime number equal 
or larger than \eqn{d}{d} and \eqn{u}{u} is an exponent, usually 2.
}
\usage{
faureprimeDesign(dimension, u = 2, range = c(0, -1))
}
\arguments{
  \item{dimension}{the number of variables (< 199)}
  \item{u        }{the exponent applied to the prime number}
  \item{range    }{the scale (min and max) of the inputs. See the details for the six predefined ranges.}
}
\details{
This is a special case of \code{\link{runif.faure}} where the number of 
generated points depends exclusively on the dimension and the selected exponent. 
For the exponent \eqn{u=2}{u=2}, the design is orthogonal and has resolution
4. It is a perfect grid \eqn{(p-1)(p+1)}{(p-1)(p+1)} on each pair of 
variables where \eqn{p}{p} is the first prime number equal or 
larger than the dimension \eqn{d}{d}. 


Six domain ranges are predefined and cover most applications:
\itemize{
  \item{\code{c(0, 0)}  corresponds to \eqn{[0, n]^d}{[0, n]^d}.}
  \item{\code{c(1, 1)}  corresponds to \eqn{[1-n, n-1]^d = [2-p^u, p^u -2]^d}{[1-n, n-1]^d = [2-p^u, p^u -2]^d}.}
  \item{\code{c(0, 1)}  corresponds to \eqn{[0, 1]^d}{[0, 1]^d}.}
  \item{\code{c(0,-1)}  corresponds to \eqn{[p^{-u}, 1-p^{-u}]^d}{[p^{-u}, 1-p^{-u}]^d}.}
  \item{\code{c(-1,-1)} corresponds to \eqn{[-1+2p^{-u}, 1-2p^{-u}]^d}{[-1+2p^{-u}, 1-2p^{-u}]^d}.}
  \item{\code{c(-1, 1)} corresponds to \eqn{[-1, 1]^d}{[-1, 1]^d}.}
}
}

\value{
\code{faureprimeDesign} returns a list with the following components:
\itemize{
  \item{design: the design of experiments}
  \item{n: the number of experiments}
  \item{dimension: the dimension}
  \item{prime: the prime number}
  \item{u: the exponent}
}
}

\author{P. Kiener}

\references{
Faure H. (1982), Discrepance de suites associees a un systeme de numeration 
(en dimension s), \emph{Acta Arith.}, 41, 337-351.

Owen A.B. (2020), On dropping the first Sobol point, 
\url{https://arxiv.org/abs/2008.08051}.
}

\examples{
## Range c(0,-1) returns the design produced by runif.faure()
plan1 <- runif.faure(n = 24, dimension = 5)$design ; plan1
plan2 <- faureprimeDesign(dimension = 5, range = c(0,-1))$design ; plan2
all.equal(plan1, plan2, tolerance = 1e-15)

## Range c(0,0) returns the original sequence of integers.
## The first (p-1) lines are on the first diagonal.
## The remaining lines are LHSs grouped in p-1 blocks of p rows.
d <- p <- 5
plan <- faureprimeDesign(dimension = d, range = c(0,0))$design ; plan
apply(plan, 2, sort)

## A regular grid (p-1)x(p+1) rotated by a small angle
pairs(plan)

plot(plan[,1], plan[,2], las = 1)
points(plan[1:(p-1),1], plan[1:(p-1),2], pch = 17, cex = 1.6)
abline(v = plan[1:(p-1),1], col = 4)

## Designs of dimensions 24x5 in various ranges
lstrg <- list(p0p0 = c(0,0),  p1p1 = c(1,1),   p0p1 = c(0,1),
              p0m1 = c(0,-1), m1m1 = c(-1,-1), m1p1 = c(-1,1))
lst <- lapply(lstrg, function(rg) faureprimeDesign(
              dimension = 5, u = 2, range = rg)$design)
lapply(lst, tail)
sapply(lst, range)

## The odd designs (p1m1, m1m1, m1p1) are orthogonal and have resolution 4.
library(lattice)

mat <- lst$m1m1 ; colnames(mat) <- LETTERS[1:5]
fml <- ~ (A+B+C+D+E)^2+I(A^2)+I(B^2)+I(C^2)+I(D^2)+I(E^2)
mmm <- model.matrix(fml, data = as.data.frame(mat))[,-1] ; tail(mmm)
cmm <- round(cov2cor(crossprod(mmm)), 3) ; cmm
lattice::levelplot(cmm[, ncol(cmm):1], at = seq(-1, 1, length.out = 10),
                   col.regions = rev(grDevices::hcl.colors(9, "PuOr")))
}

\keyword{design}

\name{nolhdrDesign}
\alias{nolhdrDesign}

\title{De Rainville's Nearly Orthogonal Latin Hypercube Designs}
\description{
     This function generates a NOLH design of dimension 2 to 29 and normalizes it to the selected range. From 2 to 7 input variables, the design is extracted from Cioppa's \code{\link{NOLHdesigns}} list and from 8 to 29 input variables it is extracted from De Rainville's \code{\link{NOLHDRdesigns}} list.
      
}
\usage{
nolhdrDesign(dimension, range = c(0, 1))
}

\arguments{
  \item{dimension }{ number of input variables }
  \item{range     }{ the scale (min and max) of the inputs. Range (0, 0) and (1, 1) are special cases and call integer ranges \eqn{(-m, m)} and \eqn{(0, 2m)}. See the examples}
}

\value{
A list with components:
  \item{n              	}{the number of lines/experiments}
  \item{dimension      	}{the number of columns/input variables}
  \item{design  		}{the design of experiments}
}

\author{T.M. Cioppa and F.-M. De Rainville for the designs. P. Kiener for the R code.}

\seealso{
De Rainville's list \code{\link{NOLHDRdesigns}}. Other NOLH or OLH designs: \code{\link{nolhDesign}}, \code{\link{olhDesign}}.
}

\examples{
## Classical normalizations
nolhdrDesign(8, range = c(1, 1))
nolhdrDesign(8, range = c(0, 1))
nolhdrDesign(8, range = c(0, 0))
nolhdrDesign(8, range = c(-1, 1))

## Change the dimnames, adjust to range (-10, 10) and round to 2 digits
xDRDN(nolhdrDesign(8), letter = "T", dgts = 2, range = c(-10, 10))

## A list of designs
lapply(5:9, function(n) nolhdrDesign(n, range = c(-1, 1))$design)
}

\keyword{design}
